<?php
	// File        : /include/lib-account.inc.php
	//
	// Description : This library is for administrator and normal user account mangement.


	// Include the configuration file.
	include_once ( $CONF_CONFIG_PATH.'/config.inc.php' );
	
	// Include the main include file for running all routine.
	include_once ( $CONF_INCLUDE_PATH.'/library.inc.php' );

	// Include the ADOdb Class
	include_once ( $CONF_INCLUDE_PATH.'/adodb/adodb.inc.php');


	// --------------------------------------------------------------------------------------- //
	// Class :: Admin_Account                                                                  //
	// --------------------------------------------------------------------------------------- //
	// Description  : This class contain the method to access the database to check,           //
	//                the administrator account for login process. And then can get            //
	//                the account information from database                                    //
	// --------------------------------------------------------------------------------------- //
	// Variable     :   $debug=0;                   // Change to 1 to enable debug mode        //
	// (Public)         $loginID;                   // login id                                //
	//                  $loginPwd;                  // login password                          //
	//                  $admin_info;                // login success, return information       //
	//                  $error;                     // Error message                           //
	//                  $autoLog=1;                 // '1'=Auto save LOG data, 0=Off           //
	//                                                                                         //
	//                  $readArray=array();         // All simular record in admin table       //
	//                  $selectLimit=0;             // Selecting limit (only work for find())  //
	//                                              // -- 0 mean no selection limit            //
	//                  $recordNum;                 // Total number of the record are founded. //
	//                  $searchRegular='AND';       // Search regular accept ('AND','OR only)  //
	//                  $searchMode='LIKE';         // Search mode accept ('LIKE', '=' only)   //
	//                  $orderBy='id,adm_login_id'; // Order by which column                   //
	//                  $orderMode='DESC';          // DESC = Descending Order                 //
	//                                              // ASC  = Ascending Order                  //
	//                                                                                         //
	// (Private)        $_DB_TBL='adm_users';       // Default table for admin account         //
	//                  $_DB_TYPE;                  // database information                    //
	//                  $_DB_ADDR;                                                             //
	//                  $_DB_PORT;                                                             //
	//                  $_DB_NAME;                                                             //
	//                  $_DB_USER;                                                             //
	//                  $_DB_PWD;                                                              //
	//                  $_LAST_QUERY;               // The last query string                   //
	//                  $_LAST_RECORD=array();      // Temp. var store the last finded record  //
	//                  $_selectOffset=0;           // Offset (only work for find())           //
	// --------------------------------------------------------------------------------------- //
	// Method       :   Admin_Account( $ID='', $PWD='' );                                      //
	// (Public)         setAdmin( $ID, $PWD );                                                 //
	//                  error();                                                               //
	//                  getData();                                                             //
	//                  read( $admID );                                                        //
	//                  find( $parent, $admLoginID, $admName, $admDesc, $admEmail, $admGroup,  //
	//                        $createDateTime, $expireDateTime, $orderBy );                    //
	//                  newAdmin( $record=array() );                                           //
	//                  update( $ID='', $recordInfo=array() );                                 //
	//                  delAccount( $ID='' );                                                  //
	//                  chkLogin( $ID, $PWD );                                                 //
	//                                                                                         //
	// (Private)        _chkLogin();                                                           //
	//                  _getRecord( $admID, $parent, $admLoginID, $admName, $admDesc,          //
	//                              $admEmail, $admGroup, $createDateTime, $expireDateTime,    //
	//                              $orderBy );                                                //
	//                  _Record( $mode=1, $admRecord=array() );                                //
	// --------------------------------------------------------------------------------------- //
	// Examples     : (Check administrator login)                                              //
	//                  $admAccount = new Admin_Account();                                     //
	//                  $admAccount->setAdmin( $ID, $PWD );                                    //
	//                  $result     = $admAccount->chkLogin();                                 //
	//                                                                                         //
	//                (other method to check login)                                            //
	//                  $admAccount = new Admin_Account( $ID, $PWD );                          //
	//                  $result     = $admAccount->chkLogin();                                 //
	//                                                                                         //
	//                (other method to check login)                                            //
	//                  $admAccount = new Admin_Account;                                       //
	//                  $result     = $admAccount->chkLogin( $ID, $PWD );                      //
	// --------------------------------------------------------------------------------------- //
	class Admin_Account
	{
		// -- Variable declaration --
		var		$debug=0;					// Change to 1 to enable debug mode
		var		$loginID;					// login id
		var		$loginPwd;					// login password
		var		$admin_info;				// login success, return information
		var		$error;						// Error message
		var		$autoLog=1;					// '1'=Auto save LOG data, 0=Off

		var		$readArray=array();			// All simular record in admin table
		var		$selectLimit=0;				// Selecting limit (only work for find())
		                                    // -- 0 mean no selection limit
		var		$recordNum;					// Total number of the record are founded.
		var		$searchRegular='AND';		// Search regular accept ('AND','OR only)
		var		$searchMode='LIKE';			// Search mode accept ('LIKE', '=' only)
		var     $orderBy='id,adm_login_id';	// Order by which column
		var		$orderMode='DESC';			// DESC = Descending Order
											// ASC	= Ascending Order


		var		$_DB_TBL='adm_users';		// Default table for admin account
		var		$_DB_TYPE;					// database information
		var		$_DB_ADDR;
		var		$_DB_PORT;
		var		$_DB_NAME;
		var		$_DB_USER;
		var		$_DB_PWD;
		var		$_LAST_QUERY;				// The last query string
		var		$_LAST_RECORD=array();		// The temp. var store the last finded record
		var		$_selectOffset=0;			// Offset (only work for find())
		

		// ===================================================================
		// == Constructor		Admin_Account::Admin_Account				==
		// ==																==
		// == Usage : $admAcc = new Admin_Account( $ID, $PWD );				==
		// == (OR)	: $admAcc = new Admin_Account();						==
		// ===================================================================
		function Admin_Account( $ID='', $PWD='' )
		{
			$this->setAdmin ( $ID, $PWD );

			GLOBAL
				$CONF_DATABASE_TYPE,
				$CONF_DATABASE_ADDR,
				$CONF_DATABASE_PORT,
				$CONF_DATABASE_DBNAME,
				$CONF_DATABASE_ID,
				$CONF_DATABASE_PWD;

			// get the database information from configuration file
			$this->_DB_TYPE = $CONF_DATABASE_TYPE;
			$this->_DB_ADDR = $CONF_DATABASE_ADDR;
			$this->_DB_PORT = $CONF_DATABASE_PORT;
			$this->_DB_NAME = $CONF_DATABASE_DBNAME;
			$this->_DB_USER = $CONF_DATABASE_ID;
			$this->_DB_PWD  = $CONF_DATABASE_PWD;

			// Check the current magic_quotes setting
			if ( get_magic_quotes_gpc() == 0 )
			{
				// Un-quote string (by default, magic_quotes is ON)
				$this->loginID	= addslashes ( $ID );
				$this->loginPwd	= addslashes ( $PWD );
			}

			// Display debug message
			if ( $this->debug )
			{
				echo 'Class -= Admin Account =- ....<br>';
				echo 'Admin loginID = '.$this->loginID."<br>\n";
				echo 'Admin loginPwd = '.$this->loginPwd."<br>\n";
				echo '_DB_TYPE = '.$this->_DB_TYPE."<br>\n";
				echo '_DB_ADDR = '.$this->_DB_ADDR."<br>\n";
				echo '_DB_PORT = '.$this->_DB_PORT."<br>\n";
				echo '_DB_NAME = '.$this->_DB_NAME."<br>\n";
				echo '_DB_USER = '.$this->_DB_USER."<br>\n";
				echo '_DB_PWD = '.$this->_DB_PWD."<br><br>\n";
			}
		}
		// -- End of constructor --

		
		// ===================================================================
		// == Class function	Admin_Account::setAccount( $ID, $PWD );     ==
		// ==																==
		// == Set the administrator ID and password.						==
		// == Usage : setAccount( $ID, $PWD );								==
		// == (OR)	: $admAcc	   = new Admin_Account();					==
		// ==		  $loginResult = $admAcc->setAccount( $ID, $PWD );		==
		// ===================================================================
		function setAdmin( $ID, $PWD )
		{
			// Check the current magic_quotes setting
			if ( get_magic_quotes_gpc() == 0 )
			{
				// Un-quote string (by default, magic_quotes is ON)
				$this->loginID	= addslashes ( $ID );
				$this->loginPwd	= addslashes ( $PWD );
			}

			// Encrypt the entered password
			$this->loginPwd = encryptString ($this->loginPwd, 250);

			// Display debug message
			if ( $this->debug )
			{
				echo 'Change to admin....<br>';
				echo "&nbsp;&nbsp;&nbsp;&nbsp; loginID = ".$this->loginID."<br>\n";
				echo "&nbsp;&nbsp;&nbsp;&nbsp; loginPwd = ".$this->loginPwd."<br>\n";
			}
		}
		// -- End of class function :: setAdmin(..) --


		// ===================================================================
		// == Class function	Admin_Account::error();  					==
		// ==																==
		// == Return error message.											==
		// == Usage : $error = error();	    								==
		// ===================================================================
		function error()
		{
			if ( $this->error )
				// Return the error message
				return $this->error;
			else
				return FALSE;

			// -- Error message --
			// WRONG_PASSWORD  : wrong password.
			// UNKNOW_ERROR    : can't connect to DB, unknow reason.
			// NONE_OF_OUTPUT  : nothing can be output
		}
		// -- End of class function :: error() --

		
		// ===================================================================
		// == Class function	Admin_Account::getData();					==
		// ==																==
		// == Return DATA when account varified, return FLASE when have any ==
		// == error.														==
		// == Usage : $data = getData();									==
		// ===================================================================
		function getData()
		{
		// ------------------------------------------------------------------------------------------------------------- //
		// -- Example of usage                                                                                        -- //
		// ------------------------------------------------------------------------------------------------------------- //
		//	$fetchArray = $adm->getData();                                                                               //
		//	$fetchArrayFlip = array_keys($fetchArray);                                                                   //
		//	foreach ( $fetchArrayFlip as $arrayName )                                                                    //
		//	{                                                                                                            //
		//		$sess_var_name  = 'SESS_'.strtoupper( $arrayName );                                                      //
		//		$$sess_var_name = noDoubleSpace( $fetchArray[$arrayName] );                                              //
		//	}                                                                                                            //
		//                                                                                                               //
		//	echo '$SESS_ID = '.$SESS_ID.' : ('.strlen($SESS_ID).')'."<br>\n";                                            //
		//	echo '$SESS_AC_ENABLE = '.$SESS_AC_ENABLE.' : ('.strlen($SESS_AC_ENABLE).')'."<br>\n";                       //
		//	echo '$SESS_PARENT = '.$SESS_PARENT.' : ('.strlen($SESS_PARENT).')'."<br>\n";                                //
		//	echo '$SESS_ADM_LOGIN_ID = '.$SESS_ADM_LOGIN_ID.' : ('.strlen($SESS_ADM_LOGIN_ID).')'."<br>\n";              //
		//	echo '$SESS_ADM_LOGIN_PASSWD = '.$SESS_ADM_LOGIN_PASSWD.' : ('.strlen($SESS_ADM_LOGIN_PASSWD).')'."<br>\n";  //
		//	echo '$SESS_ADM_NAME = '.$SESS_ADM_NAME.' : ('.strlen($SESS_ADM_NAME).')'."<br>\n";                          //
		//	echo '$SESS_ADM_DESCRIBE = '.$SESS_ADM_DESCRIBE.' : ('.strlen($SESS_ADM_DESCRIBE).')'."<br>\n";              //
		//	echo '$SESS_ADM_EMAIL = '.$SESS_ADM_EMAIL.' : ('.strlen($SESS_ADM_EMAIL).')'."<br>\n";                       //
		//	echo '$SESS_ADM_GROUP = '.$SESS_ADM_GROUP.' : ('.strlen($SESS_ADM_GROUP).')'."<br>\n";                       //
		//	echo '$SESS_DEFAULT_LANG = '.$SESS_DEFAULT_LANG.' : ('.strlen($SESS_DEFAULT_LANG).')'."<br>\n";              //
		//	echo '$SESS_DEFAULT_THEMES = '.$SESS_DEFAULT_THEMES.' : ('.strlen($SESS_DEFAULT_THEMES).')'."<br>\n";        //
		//	echo '$SESS_CREATE_DATE = '.$SESS_CREATE_DATE.' : ('.strlen($SESS_CREATE_DATE).')'."<br>\n";                 //
		//	echo '$SESS_EXPIRE_DATE_TIME = '.$SESS_EXPIRE_DATE_TIME.' : ('.strlen($SESS_EXPIRE_DATE_TIME).')'."<br>\n";  //
		//	echo '$SESS_IP_ACCESS_LIST = '.$SESS_IP_ACCESS_LIST.' : ('.strlen($SESS_IP_ACCESS_LIST).')'."<br>\n";        //
		//	echo '$SESS_IP_DENY_LIST = '.$SESS_IP_DENY_LIST.' : ('.strlen($SESS_IP_DENY_LIST).')'."<br>\n";              //
		//	echo '$SESS_MODULES_ALLOW = '.$SESS_MODULES_ALLOW.' : ('.strlen($SESS_MODULES_ALLOW).')'."<br>\n";           //
		// ------------------------------------------------------------------------------------------------------------- //
			
			if ( $this->admin_info )
				return $this->admin_info;
			else
			{
				if ( $this->debug )
					echo "Error : None of record.<br>\n";

				$this->error = 'NONE_OF_OUTPUT';
				return FALSE;
			}
		}	
		// -- End of class function :: getData() --


		// ===================================================================
		// == Class function    Admin_Account::read( $admID='' )            ==
		// ==                                                               ==
		// == Get the unique administrator record from database.            ==
		// == Usage : $admAcc->read( $admID );                              ==
		// ===================================================================
		function read( $admID )
		{
			// Save the current offset number
			$_selectOffset = $this->_selectOffset;

			// Get the result
			@ $this->_getRecord( $admID );

			// Reset the offset number
			$this->_selectOffset = $_selectOffset;

			// return the result
			return $this->readArray;
		}
		// -- End of class function :: read(..) --


		// ===================================================================
		// == Class function                                                ==
		// == Admin_Account::find( $logCode, $logDateTime,                  ==
		// ==                      $logUserID, $orderBy )                   ==
		// ==                                                               ==
		// == Get all the simular log event from the database.              ==
		// == Usage : $admAcc->find( $logCode, $logDateTime, $logUserID );  ==
		// ===================================================================
		function find( $parent='', $admLoginID='', $admName='',
	                   $admDesc='', $admEmail='', $admGroup='', 
					   $createDateTime='', $expireDateTime='', $orderBy='' )
		{
			if ( $orderBy )
				$this->orderBy = $orderBy;

			@ $this->_getRecord( '', $parent, $admLoginID, $admName,
								 $admDesc, $admEmail, $admGroup, $createDateTime,
								 $expireDateTime, $orderBy);
			return $this->readArray;
		}
		// -- End of class function :: find(..) --


		// ===================================================================
		// == Class function	                                            ==
		// == Admin_Account::newAdmin( $record=array() );                   ==
		// ==																==
		// == Create a new administrator account. The data is contain by an ==
		// == array $record. When successful create the accont, it will     ==
		// == return the new record ID. Return FALSE if any error occur.    ==
		// ==																==
		// == Usage : $result = $myGroup->newAdmin( $record=array() );      ==
		// ===================================================================
		function newAdmin( $record=array() )
		{
			if ( !is_array($record) || !count($record) )
			{
				// Check connection
				$this->error = "NO_RECORD_ADDED";

				if ( $this->debug )
					// In Debug Mode
					echo "Create a new administrator account fail<br>\n";

				// Return FALSE
				return FALSE;
			}

			if ( $this->debug )
			{
				// In Debug Mode
				echo "Create a new administrator account <br>\n";
				foreach ( array_keys($record) as $rec )
					echo '$record['.$rec.'] = '.$record[$rec]."<br>\n";
				echo "<br>\n";
			}

			// The ID is set by program, not allow set by user
			unset ($record['id']);

			if ( $record['create_date'] == '' )
				$record['create_date'] = date('Y-m-d H:i:s');

			// Set the expire date time to infinit when not provided
			if ( $record['expire_date_time'] == '' )
				$record['expire_date_time'] = '9999-12-31 23:59:59';

			// $mode = 1     ->   Update record
			// $mode = 2     ->   Add a new record
			// $mode = 3     ->   Remove record
			@$result = $this->_Record( 2, $record );

			if ( $this->debug )
				// In Debug Mode
				echo 'Inseart new record result : '.$result." (0=Fail, other is success) <br>\n";

			return $result;
		}


		// ===================================================================
		// == Class function	                                            ==
		// == OESgroup::update( $GID, $group_name, $group_describe,         ==
		// ==                   $group_enable );                            ==
		// ==																==
		// == Update a exist record. The result only be TRUE or FALSE.      ==
		// == Usage : $result = $myGroup->update( $GID, $group_name,        ==
		// ==                   $group_describe, $group_enable );           ==
		// ===================================================================
		function update( $ID='', $recordInfo=array() )
		{
			if ( !$this->read($ID) )
			{
				if ( $this->debug )
					// In Debug Mode
					echo "Update Fail : Invalid Group ID!! <br>\n";

				// Invalid Group ID
				$this->error = 'CANNOT_UPDATE';

				return FALSE;
			}
			else
			{
				// Do not allow to update the field (ID, parent and create_date)
				unset ($recordInfo['id']);
				unset ($recordInfo['parent']);
				unset ($recordInfo['create_date']);
				$recordInfo['id'] = $ID;

				// $mode = 1     ->   Update record
				// $mode = 2     ->   Add a new record
				// $mode = 3     ->   Remove record
				@$result = $this->_Record( 1, $recordInfo );

				if ( $this->debug )
					// In Debug Mode
					echo 'Update record : '.$result." (0=Fail, 1=success) <br>\n";

				return $result;
			}
		}


		// ===================================================================
		// == Class function	                                            ==
		// == OESgroup::delAccount( $ID='' );                               ==
		// ==																==
		// == Update a exist record. The result only be TRUE or FALSE.      ==
		// ==																==
		// == Usage : $result = $admAcc->delAccount( $ID='' );              ==
		// ===================================================================
		function delAccount( $ID='' )
		{
			// Do not allow to delete the ROOT account
			if ( !$this->read($ID) && $ID<= 1 )
			{
				if ( $this->debug )
					// In Debug Mode
					echo "Delete Fail : Invalid Administrator account ID!! <br>\n";

				// Invalid Group ID
				$this->error = 'CANNOT_DELETE';

				return FALSE;
			}
			else
			{
				// $mode = 1     ->   Update record
				// $mode = 2     ->   Add a new record
				// $mode = 3     ->   Remove record
				@$result = $this->_Record( 3, array( 'id'=>$ID ) );

				if ( $this->debug )
					// In Debug Mode
					echo "Delete record id=$ID : ".$result." (0=Fail, 1=success) <br>\n";

				return $result;
			}
		}

		// ===================================================================
		// == Class function	Admin_Account::chkLogin( $ID, $PWD );		==
		// ==																==
		// == Return TRUE if account is avaliable, otherwise return FALSE.  ==
		// == Usage : $loginResult = chkLogin( $ID, $PWD );					==
		// == (OR)	: $admAcc      = new Admin_Account( $ID, $PWD );		==
		// ==		  $loginResult = $admAcc->chkLogin();					==
		// ===================================================================
		function chkLogin( $ID='', $PWD='' )
		{
			$this->setAdmin( $ID, $PWD );

			return $this->_chkLogin();
		}	
		// -- End of class function :: chkLogin(..) --


		// ===================================================================
		// == Class private function	Admin_Account::_chkLogin();			==
		// ==																==
		// == Return TRUE if account is avaliable, otherwise return FALSE.  ==
		// == Usage : $loginResult = _chkLogin();							==
		// ===================================================================
		function _chkLogin()
		{
			if ( $this->loginID=='' or $this->loginPwd=='' )
			{
				$this->error = "WRONG_PASSWORD";

				// Display debug message
				if ( $this->debug )
				{
					echo 'Error : '.$this->error."<br>\n";
				}

				return FALSE;
			}
			else
			// Check the loginID and password from database
			{
				// Sleep for 1 secode to prevent DOS (Deny of services) attack
				// *** IMPORTANT TO IMPROVE THE SECURITY ***
				sleep(1);

				// Connect to SQL server using ADOdb library
				$db = ADONewConnection(db2adodb($this->_DB_TYPE));
				$db->debug = false;
				if ( chkDBIsODBC($this->_DB_TYPE) )
				{
					// The DB server type is ODBC, do this
					// ...................
					// ...................
				}
				else
				{
					// Check the sql server with different port
					if ( $this->_DB_PORT )
						$sqlServer = $this->_DB_ADDR.":".$this->_DB_PORT;
					else
						$sqlServer = $this->_DB_ADDR;
				}

				// If user provide the database name, auto change to that database.
				if ( $this->_DB_NAME )
					$db->database = $this->_DB_NAME;

				if ( $this->debug )
				{
					// In Debug Mode
					echo 'Database   = '.$this->_DB_NAME."<br>\n";
					echo 'SQL Server = '.$sqlServer;
				}

				// Create Connection
				@ $conn = $db->PConnect($sqlServer, $this->_DB_USER, $this->_DB_PWD);

				// Display the connection result
				if ( !$conn )
				{
					// -- Connection Error --
					// If the $db->ErrorMsg() store some error message
					if ( $db->ErrorMsg() )
						$this->error =  $db->ErrorMsg();
					else
						// Check connection
						$this->error = "UNKNOW_ERROR";

					if ( $this->debug )
					{
						// In Debug Mode
						echo 'Error = '.$this->error."<br>\n";
					}

					return FALSE;
				}
				else
				{
					// Change the fetch mode
					$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
					
					// -- Perpare the SQL query string --
					$query = "SELECT * FROM ".$this->_DB_TBL." WHERE adm_login_id='".$this->loginID.
						     "' AND adm_login_passwd='".$this->loginPwd."' AND ac_enable='Y'";

					if ( $this->debug )
					{
						// In Debug Mode
						echo "Connect Sucess!!<br>\n";
						echo "Query = $query <br>\n";
					}

					// execute the query
					if ( !$rs = $db->Execute($query) )
					{
						// Error on select from table
						$this->error = $db->ErrorMsg();

						if ( $this->debug )
						{
							// In Debug Mode
							echo 'Query execute error = '.$this->error."<br>\n";
						}

						return FALSE;						
					}
					else
					{
						// check the result
						$numrows = 0;
						$numrows = $rs->PO_RecordCount( $this->_DB_TBL );

						if ( $numrows == 1 )
						{
							// Display debug message
							if ( $this->debug )
							{
								echo 'Login Success!!'."<br>\n";
							}

							// The adminID and password verified, save the array of information
							$this->admin_info = $rs->fields;

							// Clear error
							$this->error = "";

							// Fetch the admin information into TEMP variables
							$fetchArrayFlip = array_keys($this->admin_info);
							foreach ( $fetchArrayFlip as $arrayName )
							{
								$temp_var_name  = 'TEMP_'.strtoupper( $arrayName );
								$$temp_var_name = noDoubleSpace( $this->admin_info[$arrayName] );
							}


							// Perpare the LOG (login success)
							$eventCode = 500;
							$eventMsg = "Administrator $SESS_ADM_NAME(LOGIN ID=$TEMP_ADM_LOGIN_ID) login successful";
						}
						else
						{
							// --- Login process fail ---
							$this->error = "WRONG_PASSWORD";

							// Display debug message
							if ( $this->debug )
							{
								// In Debug Mode
								echo 'Error = '.$this->error."<br>\n";
							}

							// Perpare the LOG (login failed)
							$eventCode = 502;
							$eventMsg = "Administrator login failed (id=$adminID,pwd=$adminPwd)";
						}


						// ***********************************************************************
						// Save the login process into log is not do by this chkLogin function,
						// we suggess you use this function to check the administrator account
						// is avaliable only, and use your own method to matain the login process.
						// ***********************************************************************
						//
						if ( autoLog )
						{
							// ------------- Save the event into LOG ------------
							$saveLog = new Log;
							$saveLog->debug=false;
							$saveLog->save( $eventCode, $eventMsg, $TEMP_ID );
							// ----------------------- END ----------------------
						}


						// Close the database connection
						$db->close();

						// Return true when check login success
						if ( $numrows == 1 )
							return TRUE;		// check login Success
					}
				}
			}
		}
		// -- End of private function :: _chkLogin(..) --



		// ===================================================================
		// == Class private function    Admin_Account::                     ==
		// == _getRecord( $admID, $parent, $admLoginID, $admName, $admDesc, ==
		// ==             $admEmail, $admGroup, $createDateTime,            == 
		// ==             $expireDateTime, $orderBy);                       ==
		// ==                                                               ==
		// == Private function to find all the simular record from the      ==
		// == database of administrator account table.                      ==
		// ==                                                               ==
		// == Usage : $admAcc->_getRecord( $admID, $parent, $admLoginID,    ==
		// ==                              $admName, $admDesc, $admEmail,   ==
		// ==                              $admGroup, $createDateTime,      ==
		// ==                              $expireDateTime, $orderBy);      ==
		// ==                                                               ==
		// == HITS  : $admID=''   select all simular result.                ==
		// ==         $admID='??' select only 1 record from database.       ==
		// ===================================================================
		function _getRecord( $admID='', $parent='', $admLoginID='', $admName='',
			                 $admDesc='', $admEmail='', $admGroup='', $createDateTime='',
			                 $expireDateTime='', $orderBy='' )
		{
			$db = ADONewConnection(db2adodb($this->_DB_TYPE));
			$db->debug = false;
			if ( chkDBIsODBC($this->_DB_TYPE) )
			{
				// The server type is ODBC, remove PORT setting
				// ...................
				// ..................
			}
			else
			{
				// Check the sql server with different port
				if ( $this->_DB_PORT )
					$sqlServer = $this->_DB_ADDR.":".$this->_DB_PORT;
				else
					$sqlServer = $this->_DB_ADDR;
			}

			// If user provide the database name, auto change to that database.
			if ( $this->_DB_NAME )
				$db->database = $this->_DB_NAME;

			if ( $this->debug )
			{
				// In Debug Mode
				echo "<hr>\n";
				echo 'SQL Server = '.$sqlServer;
			}

			// Create Connection
			@ $conn = $db->PConnect($sqlServer, $this->_DB_USER, $this->_DB_PWD);

			if ( !$conn )
			{
				// -- Connection Error --
				// If the $db->ErrorMsg() store some error message
				if ( $db->ErrorMsg() )
					$this->error =  $db->ErrorMsg();
				else
					// Check connection
					$this->error = "UNKNOW_ERROR";

				if ( $this->debug )
				{
					// In Debug Mode
					echo $this->error."<br>\n";
				}

				return FALSE;
			}
			else
			{
				if ( $this->debug )
				{
					// In Debug Mode
					echo "Connect Sucess!!<br>\n";
				}
				
				// Change the fetch mode
				$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
				
				// Check the current magic_quotes setting
				if ( get_magic_quotes_gpc() )
				{
					// Addslashes to prevent the special character input
					// (e.g. Chinese Input '\\\'...etc)
					$admLoginID		= addslashes ( $admLoginID );
					$admName		= addslashes ( $admName );
					$admDesc		= addslashes ( $admDesc );
				}
				else
				{
					// Addslashed Twise times
					$admLoginID		= addslashes ( $admLoginID );
					$admLoginID		= addslashes ( $admLoginID );
					$admName		= addslashes ( $admName );
					$admName		= addslashes ( $admName );
					$admDesc		= addslashes ( $admDesc );
					$admDesc		= addslashes ( $admDesc );
				}

				if ( $admID=='' )
				{
					// -- Perpare the SQL query string --
					// Select all simular result
					$query = "SELECT * FROM ".$this->_DB_TBL;
					$tail  = '';

					// If search mode is 'LIKE' then the tring to be search must
					// be wrap by '%'
					if ( strtoupper($this->searchMode) == 'LIKE' )
						$percent = '%';
					else
						$percent = '';

					// --- Check the parameter entered, to complete the tail of query ---
					if ( $parent and $tail)
						$tail .= ' '.$this->searchRegular. " parent ".$this->searchMode." '$percent".$parent."$percent'";
					elseif ( $parent )
							$tail = "parent ".$this->searchMode." '$percent".$parent."$percent'";

					if ( $admLoginID and $tail)
						$tail .= ' '.$this->searchRegular. " adm_login_id ".$this->searchMode." '$percent".$admLoginID."$percent'";
					elseif ( $admLoginID )
						$tail = "adm_login_id ".$this->searchMode." '$percent".$admLoginID."$percent'";

					if ( $admName and $tail)
						$tail .= ' '.$this->searchRegular. " adm_name ".$this->searchMode." '$percent".$admName."$percent'";
					elseif ( $admName )
						$tail = "adm_name ".$this->searchMode." '$percent".$admName."$percent'";

					if ( $admDesc and $tail)
						$tail .= ' '.$this->searchRegular. " adm_describe ".$this->searchMode." '$percent".$admDesc."$percent'";
					elseif ( $admDesc )
						$tail = "adm_describe ".$this->searchMode." '$percent".$admDesc."$percent'";

					if ( $admEmail and $tail)
						$tail .= ' '.$this->searchRegular. " adm_email ".$this->searchMode." '$percent".$admEmail."$percent'";
					elseif ( $admEmail )
						$tail = "adm_email ".$this->searchMode." '$percent".$admEmail."$percent'";

					if ( $admGroup and $tail)
						$tail .= ' '.$this->searchRegular. " adm_group ".$this->searchMode." '$percent".$admGroup."$percent'";
					elseif ( $admGroup )
						$tail = "adm_group ".$this->searchMode." '$percent".$admGroup."$percent'";

					if ( $createDateTime and $tail)
						$tail .= ' '.$this->searchRegular. " create_date ".$this->searchMode." '$percent".$createDateTime."$percent'";
					elseif ( $createDateTime )
						$tail = "create_date ".$this->searchMode." '$percent".$createDateTime."$percent'";

					if ( $expireDateTime and $tail)
						$tail .= ' '.$this->searchRegular. " expire_date_time ".$this->searchMode." '$percent".$expireDateTime."$percent'";
					elseif ( $expireDateTime )
						$tail = "expire_date_time ".$this->searchMode." '$percent".$expireDateTime."$percent'";
					// --- End of tail of query checking ---


					// If any parameter entered, include the tail part.
					if ( $tail )
						$query .= " WHERE ( $tail ) ";

					// Order by is set, generate the order by query string
					// (*** Hits: The order by is a list of the table column name ***)
					// (e.g orderBy='id,adm_login_id,adm_name......';)
					if ( $orderBy )
					{
						$orderArray = explode( ',', $orderBy );
						$orderBy    = '';
						for ( $i=0; $i<count($orderArray); $i++ )
						{
							$orderBy .= ' '.$orderArray[$i].' '.$this->orderMode;
							if ( ($i+1)<count($orderArray) )
								$orderBy .= ',';
						}
					}
					else
						// Use the default order by setting
						$orderBy = $this->orderBy;

					$query .= ' ORDER BY '.$orderBy;
				}
				else
				{
					// -- Perpare the SQL query string --
					// Select only 1 record by the $admID
					$query = "SELECT * FROM ".$this->_DB_TBL." WHERE id='".$admID."'";
				}

				if ( $this->debug )
				{
					// In Debug Mode
					echo 'Query = '.$query."<br>\n";
				}

				// Store the query string into $_LAST_QUERY
				$this->_LAST_QUERY = $query;

				// Clear the array
				$this->readArray = array();

				// Check using SelectLimit or Execute mode
				if ( $this->selectLimit )
				{
					// Checking the selection is the same as before, 
					// if any of the selection condition is changed, 
					// reset the offset pointer.
					if ( $this->_LAST_RECORD['parent'] != $parent || 
						 $this->_LAST_RECORD['admLoginID'] != $admLoginID ||
						 $this->_LAST_RECORD['admName'] != $admName ||
						 $this->_LAST_RECORD['admDesc'] != $admDesc ||
						 $this->_LAST_RECORD['admEmail'] != $admEmail ||
						 $this->_LAST_RECORD['admGroup'] != $admGroup ||
						 $this->_LAST_RECORD['createDateTime'] != $createDateTime ||
						 $this->_LAST_RECORD['expireDateTime'] != $expireDateTime )
					{
						// reset the $_selectOffset
						$this->_selectOffset = 0;
						$this->_LAST_RECORD['parent'] = $parent;
						$this->_LAST_RECORD['admLoginID'] = $admLoginID;
						$this->_LAST_RECORD['admName'] = $admName;
						$this->_LAST_RECORD['admDesc'] = $admDesc;
						$this->_LAST_RECORD['admEmail'] = $admEmail;
						$this->_LAST_RECORD['admGroup'] = $admGroup;
						$this->_LAST_RECORD['createDateTime'] = $createDateTime;
						$this->_LAST_RECORD['expireDateTime'] = $expireDateTime;
					}

					// SelectLimit
					@ $rs = $db->SelectLimit( $query, $this->selectLimit, 
						                      $this->_selectOffset );

					// Change the offset pointer
					$this->_selectOffset += $this->selectLimit;
				}
				else
				{
					// reset the $_selectOffset
					$this->_selectOffset = 0;

					// execute the query
					@ $rs = $db->Execute($query);
				}


				if ( !$rs )
				{
					// Error on select from table
					$this->error = $db->ErrorMsg();

					if ( $this->debug )
					{
						// In Debug Mode
						echo 'Error = '.$this->error."<br>\n";
					}

					return FALSE;						
				}
				else
				{
					// Get total number of record founded
					// *** REMEMBER!! This will not be affected by select limit ***
					@ $rs_temp = $db->Execute($query);
					$this->recordNum = $rs_temp->PO_RecordCount();

					// check the number of result
					$numrows = 0;
					$numrows = $rs->PO_RecordCount($this->_DB_TBL);
					
					if ( !$numrows )
					{
						// Can't fount any simular log event from database
						$this->error = "NO_RECORD";
					}
					else
					{
						for ( $i=0; $i<$numrows; $i++ )
						{
							// Successful retrival the data
							$this->readArray[$i] = $rs->fields;

							$rs->MoveNext();
						}
					}


					// Display debug message
					if ( $this->debug )
					{
						echo 'Total number of record = '.intval($this->recordNum)."<br>\n";
						if ( $this->selectLimit )
							echo 'Select Limit = '.$this->selectLimit."<br>\n";
						echo 'Fetch array = ';
						
						if ( ( $this->recordNum > 1 ) && 
							 (($this->_selectOffset - $this->selectLimit) < intval($this->recordNum) ) )
						{
							echo ($this->_selectOffset - $this->selectLimit).' to ';
							if ( ( $this->selectLimit ) &&
								 (($this->_selectOffset - 1) <= intval($this->recordNum) ) )
								echo ($this->_selectOffset - 1)."<br>\n";
							else
								echo $this->recordNum."<br>\n";
						}
						else
							echo "0<br>\n";

							echo '$this->_selectOffset = '.$this->_selectOffset ."<br>\n";
							echo '$this->selectLimit = '.$this->selectLimit."<br>\n";

						for ( $i=0; $i<count($this->readArray); $i++ )
						{
							$fetchArray = $this->readArray[$i];
							$fetchArrayFlip = array_keys($fetchArray);
							foreach ( $fetchArrayFlip as $arrayName )
							{
								$sess_var_name  = 'TEMP_'.strtoupper( $arrayName );
								$$sess_var_name = noDoubleSpace( $fetchArray[$arrayName] );
							}
							$count = $this->_selectOffset - $this->selectLimit + $i;

							echo "<br>\n";
							echo 'array['.($count)."]['id'] = ".
								 $TEMP_ID.' : ('.strlen($TEMP_ID).')'."<br>\n";
							echo 'array['.($count)."]['ac_enable'] = ".
								 $TEMP_AC_ENABLE.' : ('.strlen($TEMP_AC_ENABLE).')'."<br>\n";
							echo 'array['.($count)."]['parent'] = ".
								 $TEMP_PARENT.' : ('.strlen($TEMP_PARENT).')'."<br>\n";
							echo 'array['.($count)."]['adm_login_id'] = ".
								 $TEMP_ADM_LOGIN_ID.' : ('.strlen($TEMP_ADM_LOGIN_ID).')'."<br>\n";
							echo 'array['.($count)."]['adm_login_passwd'] = ".
								 $TEMP_ADM_LOGIN_PASSWD.' : ('.strlen($TEMP_ADM_LOGIN_PASSWD).')'."<br>\n";
							echo 'array['.($count)."]['adm_name'] = ".
								 $TEMP_ADM_NAME.' : ('.strlen($TEMP_ADM_NAME).')'."<br>\n";
							echo 'array['.($count)."]['adm_describe'] = ".
								 $TEMP_ADM_DESCRIBE.' : ('.strlen($TEMP_ADM_DESCRIBE).')'."<br>\n";
							echo 'array['.($count)."]['adm_email'] = ".
								 $TEMP_ADM_EMAIL.' : ('.strlen($TEMP_ADM_EMAIL).')'."<br>\n";
							echo 'array['.($count)."]['adm_group'] = ".
								 $TEMP_ADM_GROUP.' : ('.strlen($TEMP_ADM_GROUP).')'."<br>\n";
							echo 'array['.($count)."]['default_lang'] = ".
								 $TEMP_DEFAULT_LANG.' : ('.strlen($TEMP_DEFAULT_LANG).')'."<br>\n";
							echo 'array['.($count)."]['default_themes'] = ".
								 $TEMP_DEFAULT_THEMES.' : ('.strlen($TEMP_DEFAULT_THEMES).')'."<br>\n";
							echo 'array['.($count)."]['create_date'] = ".
								 $TEMP_CREATE_DATE.' : ('.strlen($TEMP_CREATE_DATE).')'."<br>\n";
							echo 'array['.($count)."]['expire_date_time'] = ".
								 $TEMP_EXPIRE_DATE_TIME.' : ('.strlen($TEMP_EXPIRE_DATE_TIME).')'."<br>\n";
							echo 'array['.($count)."]['ip_access_list'] = ".
								 $TEMP_IP_ACCESS_LIST.' : ('.strlen($TEMP_IP_ACCESS_LIST).')'."<br>\n";
							echo 'array['.($count)."]['ip_deny_list'] = ".
								 $TEMP_IP_DENY_LIST.' : ('.strlen($TEMP_IP_DENY_LIST).')'."<br>\n";
							echo 'array['.($count)."]['modules_allow'] = ".
								 $TEMP_MODULES_ALLOW.' : ('.strlen($TEMP_MODULES_ALLOW).')'."<br>\n";
						}
						echo "<hr>\n";
					}


					if ( !$numrows )
						return FALSE;
					else
						return TRUE;				
				}
			}
		}
		// -- End of private function :: _getRecord(..) --


	
		// ===================================================================
		// == Class private function                                        ==
		// == OESgroup::_Record ( $mode=1, $admRecord=array() )             ==
		// ==                                                               ==
		// == Private function to create/update/remove a record from the    ==
		// == database of administrator account.                            ==
		// ==                                                               ==
		// == Usage : $admAcc->_Record( 1, array() );                       ==
		// ==                                                               ==
		// ==   $mode = 1     ->   Update record                            ==
		// ==   $mode = 2     ->   Add a new record                         ==
		// ==   $mode = 3     ->   Remove record                            ==
		// ==                                                               ==
		// ==   $admRecord['id'];                                           ==
		// ==   $admRecord['ac_enable'];                                    ==
		// ==   $admRecord['parent'];                                       ==
		// ==   $admRecord['adm_login_id'];                                 ==
		// ==   $admRecord['adm_login_passwd'];                             ==
		// ==   $admRecord['adm_name'];                                     ==
		// ==   $admRecord['adm_describe'];                                 ==
		// ==   $admRecord['adm_email'];                                    ==
		// ==   $admRecord['adm_group'];                                    ==
		// ==   $admRecord['default_lang'];                                 ==
		// ==   $admRecord['default_themes'];                               ==
		// ==   $admRecord['create_date'];                                  ==
		// ==   $admRecord['expire_date_time'];                             ==
		// ==   $admRecord['ip_access_list'];                               ==
		// ==   $admRecord['ip_deny_list'];                                 ==
		// ==   $admRecord['modules_allow'];                                ==
		// ===================================================================
		function _Record( $mode=1, $admRecord=array() )
		{
			// $mode = 1     ->   Update record
			// $mode = 2     ->   Add a new record
			// $mode = 3     ->   Remove record
			if ( $this->debug )
			{
				// In Debug Mode
				echo 'The operation mode = '.$mode." (1=Update, 2=Insert, 3=Remove) <br>\n";
				echo "The input data is listed below : <br>\n";
				foreach ( array_keys($admRecord) as $rec )
					echo '$admRecord['.$rec.'] = '.$admRecord[$rec]."<br>\n";
			}

			$db = ADONewConnection(db2adodb($this->_DB_TYPE));
			$db->debug = 0;
			if ( chkDBIsODBC($this->_DB_TYPE) )
			{
				// The server type is ODBC, remove PORT setting
				// ...................
				// ..................
			}
			else
			{
				// Check the sql server with different port
				if ( $this->_DB_PORT )
					$sqlServer = $this->_DB_ADDR.":".$this->_DB_PORT;
				else
					$sqlServer = $this->_DB_ADDR;
			}


			// If user provide the database name, auto change to that database.
			if ( $this->_DB_NAME )
				$db->database = $this->_DB_NAME;

			if ( $this->debug )
			{
				// In Debug Mode
				echo '<hr>';
				echo 'Database   = '.$this->_DB_NAME."<br>\n";
				echo 'SQL Server = '.$sqlServer;
			}


			// Create Connection
			@ $conn = $db->PConnect($sqlServer, $this->_DB_USER, $this->_DB_PWD);

			if ( !$conn )
			{
				// -- Connection Error --
				// If the $db->ErrorMsg() store some error message
				if ( $db->ErrorMsg() )
					$this->error =  $db->ErrorMsg();
				else
					// Check connection
					$this->error = "UNKNOW_ERROR";

				if ( $this->debug )
					// In Debug Mode
					echo $this->error."<br>\n";

				return FALSE;
			}
			else
			{
				if ( $this->debug )
				{
					// In Debug Mode
					echo "Connect Sucess!!<br>\n";
					echo 'get_magic_quotes_gpc() = '.get_magic_quotes_gpc()."<br>\n";
					echo 'get_magic_quotes_runtime() = '.get_magic_quotes_runtime()."<br>\n";
				}
				
				// The group_enable is only allow 'Y' or 'N'
				if ( $admRecord['ac_enable'] != 'Y' )
					$admRecord['ac_enable'] = 'N';

				// Check the current magic_quotes setting
				if ( get_magic_quotes_gpc() )
				{
					// Addslashes to prevent the special character input
					// (e.g. Chinese Input '\\\'...etc)
					foreach ( array_keys($admRecord) as $rec )
						$admRecord[$rec] = stripslashes ( $admRecord[$rec] );
				}


				// -- Perpare the SQL query string --
				// $mode = 1     ->   Update record
				// $mode = 2     ->   Add a new record
				// $mode = 3     ->   Remove record
				if ( $mode == 1 )
				{
					// --------------------------------------------- //
					// Update a record (i.e. update a existed GROUP) //
					// --------------------------------------------- //

					// Query an empty record set
					$sql = "SELECT * FROM ".$this->_DB_TBL." WHERE id = ".$admRecord['id']; 
					@ $rs = $db->Execute($sql);

					// Get the insert quert string
					@ $query = $db->GetUpdateSQL($rs, $admRecord);

					// Assume the empty query is mean no need to update
					if ( strlen($query) == 0 )
						return TRUE;
				}
				elseif ( $mode == 2 )
				{
					// --------------------------------------------- //
					// Add a new record (i.e. create a new GROUP)    //
					// --------------------------------------------- //

					// Get the next avaliable ID and the current date and time
					$admRecord['id']          = $db->GenID($this->_DB_TBL.'_id_seq');

					// Query an empty record set
					$sql = "SELECT * FROM ".$this->_DB_TBL." WHERE id = -1";
					@ $rs = $db->Execute($sql);

					// Get the insert quert string
					@ $query = $db->GetInsertSQL($rs, $admRecord);
				}
				elseif ( $mode == 3 )
				{
					// --------------------------------------------- //
					// Remove a record (i.e. remove a GROUP)         //
					// --------------------------------------------- //
					$id = $admRecord['id'];
					$query = "DELETE FROM ".$this->_DB_TBL." WHERE id='$id'";
				}

				if ( $this->debug )
					// In Debug Mode
					echo 'Query = '.$query."<br>\n";

				// Store the query string into $_LAST_QUERY
				$this->_LAST_QUERY = $query;

				// execute the query
				if ( @!$db->Execute($query) )
				{
					// Can't inseart a new group
					// If the $db->ErrorMsg() store some error message
					if ( $db->ErrorMsg() )
						$this->error =  $db->ErrorMsg();
					else
					{
						if ( $mode == 1 )
							// Can't update
							$this->error = "CANNOT_UPDATE";
						elseif ( $mode == 2 )
							// Can't inseart
							$this->error = "CANNOT_INSERT";
						elseif ( $mode == 3 )
							// Can't delete
							$this->error = "CANNOT_DELETE";
					}

					if ( $this->debug )
						// In Debug Mode
						echo 'Error = '.$this->error."<br><hr>\n";

					return FALSE;						
				}
				else
				{
					// ************************************** //
					// ************************************** //
					// ************************************** //
					// Save the event into log
					// ************************************** //
					// ************************************** //
					// ************************************** //

					if ( $mode == 2 )
						// Inseart a new group SUCCESS!!
						// return the new group id
						return $admRecord['id'];
					elseif ( $mode == 1 || $mode == 3 )
						return TRUE;
				}

			}
		}
		// -- End of private function :: _Record(..) --	
	
	}
	// -- End of class :: Admin_Account --
?>