<?php
	// File        : /include/lib-file.inc.php
	//
	// Description : This library contain all the class or function for file handling.


	// Include the configuration file.
	include_once ( $CONF_CONFIG_PATH.'/config.inc.php' );
	
	// Include the main include file for running all routine.
	include_once ( $CONF_INCLUDE_PATH.'/library.inc.php' );

	// Include the MIME defined file type.
	include_once ( $CONF_INCLUDE_PATH.'/lib-mime.inc.php' );

	// Include the timer class.
	include_once ( $CONF_INCLUDE_PATH.'/lib-time.inc.php' );

	// Include the ADOdb Class
	include_once ( $CONF_INCLUDE_PATH.'/adodb/adodb.inc.php');



	// ------------------------------------------------------------------------------------ //
	// Class :: myFile                                                                      //
	// ------------------------------------------------------------------------------------ //
	// Description  : This class contain the method to access file system indirectly.       //
	//                This is vary useful to protect the file in system and only allow      //
	//                the authorise person to download the file.                            //
	// ------------------------------------------------------------------------------------ //
	// Variable     :   $filename;                  // the file will be sent                //
	//                  $error=0;                   // set to 1 to enable error mode        //
	//                  $trafControl=0;             // set to 1 to enable traffic control   //
	//                  $noCache=0;                 // set to 1 to disable using cache      //
	//                  $trafSpeed=10240;			// speed byte per second (byte/s)       //
	//                                              // default is 10240 = 10KB/s            //
	//                  $_trafIntrval=100;			// default interval time is 100us       //
	// ------------------------------------------------------------------------------------ //
	// Method       :   myFile( $inFileName = '' );                                         //
	// (Public)         setFile( $inFileName = '' );                                        //
	//                  getFile();                                                          //
	//                  loadFile();                                                         //
	// ------------------------------------------------------------------------------------ //
	// Examples     : (Create a myfile object)                                              //
	//                  $myfile     = new myFile();                                         //
	//                                                                                      //
	//                (other method to create myfile object)                                //
	//                  $myfile     = new myFile();                                         //
	//                  $myfile->setFile ($file );                                          //
	//                                                                                      //
	//                (force browser to download file)                                      //
	//                  $myfile->getFile();                                                 //
	//                                                                                      //
	//                (force browser to display the content of file)                        //
	//                  $myfile->loadFile();                                                //
	//                                                                                      //
	//                (Limit the bandwidth)                                                 //
	//                  $myFile->trafControl=1;         // enable bandwidth limit           //
	//                  $myFile->trafBuffer=1024;       // buffer size                      //
	//                  $myFile->trafInterval=30000;    // interval time                    //
	// ------------------------------------------------------------------------------------ //
	class myFile
	{
		// -- Variable declaration --
		var		$filename;					// the file will be sent
		var		$error=0;					// set to 1 to enable error mode
		var		$trafControl=0;				// set to 1 to enable traffic control
		var		$noCache=0;					// set to 1 to disable using cache
		var		$trafSpeed=10240;			// speed byte per second (byte/s)
											// default is 10240 = 10KB/s
		var		$_trafIntrval=100;			// default interval time is 100us



		// ===================================================================
		// == Class function    myFile::setFile( $inFileName = '' );        ==
		// ==                                                               ==
		// == Usage : $myfile = new myFile();                               ==
		// == (OR)	: $myfile = new myFile( './abc.txt' );                  ==
		// ===================================================================
		function myFile( $inFileName = '' )
		{
			$this->filename = $inFileName;
		}
		// -- End of constructor --



		// ===================================================================
		// == Class function    myFile::setFile( $inFileName = '' );        ==
		// ==                                                               ==
		// == Set the filename.                                             ==
		// == Usage : $myfile->setFile( './abc.txt' );                      ==
		// ===================================================================
		function setFile( $inFileName = '' )
		{
			$this->filename = $inFileName;
		}
		// -- End of class function :: setFile(..) --



		// ===================================================================
		// == Class function	myFile::getFile();                          ==
		// ==																==
		// == This function will force the user to download the file.       ==
		// == *** WARNING!! This function will automatic sending the header ==
		// == information to client browser, be sure don't send anything to ==
		// == client bworser before use this function.                      ==
		// ==																==
		// == Usage : $myfile = new myFile();								==
		// == (OR)	: $myfile = new myFile( './abc.txt' );					==
		// ===================================================================
		function getFile()
		{
			// Fail if the file not found
			if ( !is_file( $this->filename ) )
			{
				if ( $this->error )
				{
					// Tell the browser file not found
					header("Cache-control: private");
					header("Content-Type: text/html" );
					header("Status: 404 Not Found");
					header("HTTP/1.0 404 Not Found");
					header("HTTP 404 Not Found");
					header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); 
					header("Cache-Control: no-store, no-cache, must-revalidate");	// HTTP 1.1
					header("Cache-Control: post-check=0, pre-check=0", false);
					header("Pragma: no-cache");										// HTTP 1.0
					echo "HTTP 404 Not Found\n";
				}
				return FALSE;
			}
			
			// Get the base filename without the path
			$filename = basename( $this->filename );

			// Get the path name without the filename
			$path     = dirname( $this->filename );

			// Sending the header information 
			if ( $this->noCache ) header("Cache-control: private");
			if ( $this->noCache ) header("Cache-Control: no-store, no-cache, must-revalidate");		// HTTP 1.1
			if ( $this->noCache ) header("Cache-Control: post-check=0, pre-check=0", false);
			if ( $this->noCache ) header("Pragma: no-cache");										// HTTP 1.0
			header("Content-type: application/octet-stream");
			header("Content-Disposition: attachment; filename=$filename");
			header("Content-transfer-encoding: binary");
			header("Content-Length: ".filesize( $this->filename ));

			// Use output the binary stream on browser
			$fp = fopen( $this->filename, 'r' );

			if ( !$this->trafControl )
				// No traffic limit, use passthru
				fpassthru($fp);
			else
			{
				// Traffic limited                                                                                          

				$timer = new myTimer;     // Create a timer object
				$sendByte = 0;            // clear the sended byte counter

				// To minimize the CPU sampling time not over 200 times per second
				if ( ( $this->trafSpeed / 200 ) >= 1024 )
					$bufferSize = $this->trafSpeed / 200;
				else
					$bufferSize = 1024;

				// start the timer
				$timer->start();

				// While...Loop until the end of the file
				while( !feof($fp) ) 
				{
					// Push the timer for check the time, but actually the timer
					// is also counting.
					$timer->stop();
					$timer->t_elapsed=0;

					// check the sended byte and the time not over the limit,
					// by default the system will buffer 1024 byte of data.
					if ( ($sendByte + $bufferSize) <= $this->trafSpeed && $timer->elapsed() <= 1 )
					{
						// Allow to send
						$buffer = fread( $fp, $bufferSize );
						echo $buffer;
						$sendByte += $bufferSize;

						// This is inportant to release the memory usage
						unset ($buffer);
					}
					else
					{
						$timer->stop();
						$timer->t_elapsed=0;
						if ( $timer->elapsed() <= 1 )
							usleep($this->_trafIntrval);
						else
						{
							$timer->start();
							$sendByte = 0;
						}
					}
				}
			}
		}
		// -- End of class function :: getFile(..) --




		// ===================================================================
		// == Class function	myFile::loadFile();                         ==
		// ==																==
		// == This function will try to display the file content on the     ==
		// == browser with apporate MIME type.                              ==
		// == *** WARNING!! This function will automatic sending the header ==
		// == information to client browser, be sure don't send anything to ==
		// == client bworser before use this function.                      ==
		// ==																==
		// == Usage : $myfile = new myFile();								==
		// == (OR)	: $myfile = new myFile( './abc.txt' );					==
		// ===================================================================
		function loadFile()
		{
			// Fail if the file not found
			if ( !is_file( $this->filename ) )
			{
				if ( $this->error )
				{
					// Tell the browser file not found
					header("Cache-control: private");
					header("Content-Type: text/html" );
					header("Status: 404 Not Found");
					header("HTTP/1.0 404 Not Found");
					header("HTTP 404 Not Found");
					header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); 
					header("Cache-Control: no-store, no-cache, must-revalidate");	// HTTP 1.1
					header("Cache-Control: post-check=0, pre-check=0", false);
					header("Pragma: no-cache");										// HTTP 1.0
					echo "HTTP 404 Not Found\n";
				}
				return FALSE;
			}
			
			// Get the base filename without the path
			$filename = basename( $this->filename );

			// Get the path name without the filename
			$path     = dirname( $this->filename );

			// Sending the header information 
			if ( $this->noCache ) header("Cache-control: private");
			header('Content-Type: '.getMIME($filename) );
			header("Content-Disposition: inline; filename=$filename");
			header("Content-Length: ".filesize( $this->filename ));
			header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT"); 
			if ( $this->noCache ) header("Cache-Control: no-store, no-cache, must-revalidate");		// HTTP 1.1
			if ( $this->noCache ) header("Cache-Control: post-check=0, pre-check=0", false);
			if ( $this->noCache ) header("Pragma: no-cache");										// HTTP 1.0

			// Use output the binary stream on browser
			$fp = fopen( $this->filename, 'r' );

			if ( !$this->trafControl )
				// No traffic limit, use passthru
				fpassthru($fp);
			else
			{
				// Traffic limited

				$timer = new myTimer;     // Create a timer object
				$sendByte = 0;            // clear the sended byte counter

				// To minimize the CPU sampling time not over 200 times per second
				if ( ( $this->trafSpeed / 200 ) >= 1024 )
					$bufferSize = $this->trafSpeed / 200;
				else
					$bufferSize = 1024;

				// start the timer
				$timer->start();

				// While...Loop until the end of the file
				while( !feof($fp) ) 
				{
					// Push the timer for check the time, but actually the timer
					// is also counting.
					$timer->stop();
					$timer->t_elapsed=0;

					// check the sended byte and the time not over the limit,
					// by default the system will buffer 1024 byte of data.
					if ( ($sendByte + $bufferSize) <= $this->trafSpeed && $timer->elapsed() <= 1 )
					{
						// Allow to send
						$buffer = fread( $fp, $bufferSize );
						echo $buffer;
						$sendByte += $bufferSize;

						// This is inportant to release the memory usage
						unset ($buffer);
					}
					else
					{
						$timer->stop();
						$timer->t_elapsed=0;
						if ( $timer->elapsed() <= 1 )
							usleep($this->_trafIntrval);
						else
						{
							$timer->start();
							$sendByte = 0;
						}
					}
				}
			}
		}
		// -- End of class function :: setFile(..) --
	}
?>
