<?php
	// File        : /include/lib-log.inc.php
	//
	// Description : This is a CLASS of Logging system.


	// Include the configuration file.
	include_once ( $CONF_CONFIG_PATH.'/config.inc.php' );
	
	// Include the main include file for running all routine.
	include_once ( $CONF_INCLUDE_PATH.'/library.inc.php' );

	// Include the ADOdb Class
	include_once ( $CONF_INCLUDE_PATH.'/adodb/adodb.inc.php');


	// ------------------------------------------------------------------------------------ //
	// Class :: Log                                                                         //
	// ------------------------------------------------------------------------------------ //
	// Description  : This is a class of logging system. This contain a set of method to    //
	//                access the database table 'log'.                                      //
	//                The purpose of this class is used to put or read the log event into   //
	//                or from the database. If you have any problem of using this class, or //
	//                don't know how this work, you can enable the DEBUG mode and this will //
	//                display all the debug messages on the screen (english only).          //
	// ------------------------------------------------------------------------------------ //
	// Variable     :   $debug=0;               // Change to 1 to enable debug mode         //
	// (Public)         $SID;                   // Session ID                               //
	//                  $error;                 // Error message                            //
	//                  $readArray;             // All simular event from log               //
	//                  $selectLimit=0;         // Selecting limit (only work for find())   //
	//                  $recordNum;             // Total number of the record are founded.  //
	//                  $orderMode='DESC';      // DESC = Descending Order                  //
	//                                          // ASC  = Ascending Order                   //
	//                                                                                      //
	// (Private)        $_selectOffset=0;       // Offset (only work for find())            //
	//                  $_DB_TBL='log';         // Default table for logging system         //
	//                  $_DB_TYPE;              // database information                     //
	//                  $_DB_ADDR;                                                          //
	//                  $_DB_PORT;                                                          //
	//                  $_DB_NAME;                                                          //
	//                  $_DB_USER;                                                          //
	//                  $_DB_PWD;                                                           //
	//                  $_FIND_CODE;            // Temp. var used for check the selection   //
	//                  $_FIND_DATETIME;        // is the the same record                   //
	//                  $_FIND_USERID;          //                                          //
	//                  $_LAST_QUERY;           // Store the last time of query string      //
	// ------------------------------------------------------------------------------------ //
	// Method       :   Log();                                                              //
	// (Public)         save( $logCode='', $logMsg='' );                                    //
	//                  read( $logID );                                                     //
	//                  find( $logCode, $logDateTime, $logUserID );                         //
	//                  error();                                                            //
	// (Private)        _saveLog( $logID, $logMsg );                                        //
	//                  _getLog( $logID='', $logCode='', $logDateTime='', $logUserID='');   //
	// ------------------------------------------------------------------------------------ //
	// Examples     : (Save Log)                                                            //
	//                  $myLog      = new Log();                                            //
	//                  $result     = $myLog->save( $logID='', $logMsg='' );                //
	//                                                                                      //
	//                (Find all simular log event)                                          //
	//                  $myLog      = new Log();                                            //
	//                  $result     = $myLog->find( $logCode, $logDateTime, $logUserID );   //
	//                                                                                      //
	//                (get unique log record)                                               //
	//                  $myLog      = new Log();                                            //
	//                  $result     = $myLog->read( $logID );                               //
	// ------------------------------------------------------------------------------------ //
	class Log
	{
		// -- Variable declaration --
		var		$debug=0;					// Change to 1 to enable debug mode
		var		$SID;						// Session ID
		var		$error;						// Error message
		var		$readArray=array();			// All simular event from log
		var		$selectLimit=0;				// Selecting limit (only work for find())
		                                    // -- 0 mean no selection limit
		var		$recordNum;					// Total number of the record are founded.
		var     $orderBy='id,log_time';		// Order by which column
		var		$orderMode='DESC';			// DESC = Descending Order
											// ASC	= Ascending Order

		var		$_selectOffset=0;			// Offset (only work for find())
		var		$_DB_TBL='log';				// Default table for logging system
		var		$_DB_TYPE;					// database information
		var		$_DB_ADDR;
		var		$_DB_PORT;
		var		$_DB_NAME;
		var		$_DB_USER;
		var		$_DB_PWD;
		var		$_FIND_CODE;				// Temp. var used for check the selection
		var		$_FIND_DATETIME;			// -- is the the same record
		var		$_FIND_USERID;				//
		var		$_LAST_QUERY;				// Store the last time of query string


		// ===================================================================
		// == Constructor       Log::Log()                                  ==
		// ==                                                               ==
		// == Usage : $myLog = new Log();                                   ==
		// ===================================================================
		function Log()
		{
			if ( session_id() )
				$this->SID = session_id();

			GLOBAL
				$CONF_DATABASE_TYPE,
				$CONF_DATABASE_ADDR,
				$CONF_DATABASE_PORT,
				$CONF_DATABASE_DBNAME,
				$CONF_DATABASE_ID,
				$CONF_DATABASE_PWD;

			// get the database information from configuration file
			$this->_DB_TYPE = $CONF_DATABASE_TYPE;
			$this->_DB_ADDR = $CONF_DATABASE_ADDR;
			$this->_DB_PORT = $CONF_DATABASE_PORT;
			$this->_DB_NAME = $CONF_DATABASE_DBNAME;
			$this->_DB_USER = $CONF_DATABASE_ID;
			$this->_DB_PWD  = $CONF_DATABASE_PWD;
		}
		// -- End of constructor --


		// ===================================================================
		// == Class function    Log::save($logCode, $logMsg, $logUserID='') ==
		// ==                                                               ==
		// == Save the event into database table -- 'Log'.                  ==
		// == Usage : $myLog->save( $logID='', $logMsg='' );                ==
		// ===================================================================
		function save( $logCode, $logMsg, $logUserID='' )
		{
			return $this->_saveLog( $logCode, $logMsg, $logUserID );
		}
		// -- End of class function :: save(..) --



		// ===================================================================
		// == Class function    Log::read( $logID='' )                      ==
		// ==                                                               ==
		// == Get the unique log record from database.                      ==
		// == Usage : $myLog->read( $logID );                               ==
		// ===================================================================
		function read( $logID )
		{
			// Save the current offset number
			$_selectOffset = $this->_selectOffset;

			// Get the result
			@ $this->_getLog( $logID );

			// Reset the offset number
			$this->_selectOffset = $_selectOffset;

			// return the result
			return $this->readArray;
		}
		// -- End of class function :: read(..) --


		// ===================================================================
		// == Class function    Log::find( $logCode, $logDateTime,          ==
		// ==                              $logUserID, $orderBy )           ==
		// ==                                                               ==
		// == Get all the simular log event from the database.              ==
		// == Usage : $myLog->find( $logCode, $logDateTime, $logUserID );   ==
		// ===================================================================
		function find( $logCode='', $logDateTime='', $logUserID='', $orderBy='' )
		{
			if ( $orderBy )
				$this->orderBy = $orderBy;

			@ $this->_getLog( '', $logCode, $logDateTime, $logUserID, $this->orderBy );
			return $this->readArray;
		}
		// -- End of class function :: find(..) --


		// ===================================================================
		// == Class function    Log::error();                               ==
		// ==                                                               ==
		// == Return error message.                                         ==
		// == Usage : $myLog->error();                                      ==
		// ===================================================================
		function error()
		{
			if ( $this->error )
				// Return the error message
				return $this->error;
			else
				return FALSE;

			// -- Error message --
			// UNKNOW_ERROR    : can't connect to DB, unknow reason.
			// NO_RECORD       : Log not found.
			// NONE_OF_INPUT   : No imput, nothing to save into log
		}
		// -- End of class function :: error() --



		// ===================================================================
		// == Class private function    Log::_saveLog( $LogID, $logMsg );   ==
		// ==                                                               ==
		// == Private function to save the log into database.               ==
		// == Usage : $myLog->_saveLog( $logID, $logMsg );                  ==
		// ===================================================================
		function _saveLog( $logCode, $logMsg, $logUserID )
		{
			$db = ADONewConnection(db2adodb($this->_DB_TYPE));
			$db->debug = false;
			if ( chkDBIsODBC($this->_DB_TYPE) )
			{
				// The server type is ODBC, remove PORT setting
				// ...................
				// ..................
			}
			else
			{
				// Check the sql server with different port
				if ( $this->_DB_PORT )
					$sqlServer = $this->_DB_ADDR.":".$this->_DB_PORT;
				else
					$sqlServer = $this->_DB_ADDR;
			}

			// If user provide the database name, auto change to that database.
			if ( $this->_DB_NAME )
				$db->database = $this->_DB_NAME;

			if ( $this->debug )
			{
				// In Debug Mode
				echo "<hr>\n";
				echo 'Database   = '.$this->_DB_NAME."<br>\n";
				echo 'SQL Server = '.$sqlServer;
			}

			// Create Connection
			@ $conn = $db->PConnect($sqlServer, $this->_DB_USER, $this->_DB_PWD);
			
			if ( !$conn )
			{
				// -- Connection Error --
				// If the $db->ErrorMsg() store some error message
				if ( $db->ErrorMsg() )
					$this->error =  $db->ErrorMsg();
				else
					// Check connection
					$this->error = "UNKNOW_ERROR";

				if ( $this->debug )
				{
					// In Debug Mode
					echo 'Error = '.$this->error."<br>\n";
				}

				return FALSE;
			}
			else
			{
				if ( $this->debug )
				{
					// In Debug Mode
					echo "Connect Sucess!!<br>\n";
				}

				$currentDateTime = date("Y-m-d H:i:s");
				$eventCode = $logCode;
				$eventMsg = $logMsg.', session='.$this->SID;
				$nextID = $db->GenID('log_id_seq');
				$query = "INSERT INTO ".$this->_DB_TBL." VALUES ".
					     "('$nextID', '$eventCode', '$currentDateTime', ".
					     "'$eventMsg', '$logUserID')";
				
				if ( $this->debug )
				{
					// In Debug Mode
					echo 'Insert Query = '.$query."<br>\n";
				}

				// Store the query string into $_LAST_QUERY
				$this->_LAST_QUERY = $query;

				@ $result = $db->Execute($query);
			}
			$db->close();

			// Display debug data
			if ( $this->debug )
			{
				if ( $result )
					echo "Insertion LOG success!!<br><br>\n";
				else
					echo "Insertion LOG failse!!<br><br>\n";

				echo "<hr>\n";
			}			

			if ( $result )
				// Log saved.
				return TRUE;
			else
				// Can't save the Log
				return FALSE;
		}
		// -- End of private function :: _saveLog(..) --


		// ===================================================================
		// == Class private function    Log::_getLog( $logID='',            ==
		// ==                                $logCode='', $logDateTime='',  ==
		// ==                                $logUserID='', , $orderBy=''); ==
		// ==                                                               ==
		// == Private function to find the simular log event from database. ==
		// == Usage : $myLog->_getLog( $logID='', $logCode='',              ==
		// ==                          $logDateTime='', $logUserID='');     ==
		// ==                                                               ==
		// == HITS  : $logID=''   select all simular result.                ==
		// ==         $logID='??' select only 1 log event by 'LogID'        ==
		// ===================================================================
		function _getLog( $logID='', $logCode='', $logDateTime='', $logUserID='', $orderBy='')
		{
			$db = ADONewConnection(db2adodb($this->_DB_TYPE));
			$db->debug = false;
			if ( chkDBIsODBC($this->_DB_TYPE) )
			{
				// The server type is ODBC, remove PORT setting
				// ...................
				// ..................
			}
			else
			{
				// Check the sql server with different port
				if ( $this->_DB_PORT )
					$sqlServer = $this->_DB_ADDR.":".$this->_DB_PORT;
				else
					$sqlServer = $this->_DB_ADDR;
			}

			// If user provide the database name, auto change to that database.
			if ( $this->_DB_NAME )
				$db->database = $this->_DB_NAME;

			if ( $this->debug )
			{
				// In Debug Mode
				echo "<hr>\n";
				echo 'SQL Server = '.$sqlServer;
			}

			// Create Connection
			@ $conn = $db->PConnect($sqlServer, $this->_DB_USER, $this->_DB_PWD);

			if ( !$conn )
			{
				// -- Connection Error --
				// If the $db->ErrorMsg() store some error message
				if ( $db->ErrorMsg() )
					$this->error =  $db->ErrorMsg();
				else
					// Check connection
					$this->error = "UNKNOW_ERROR";

				if ( $this->debug )
				{
					// In Debug Mode
					echo $this->error."<br>\n";
				}

				return FALSE;
			}
			else
			{
				if ( $this->debug )
				{
					// In Debug Mode
					echo "Connect Sucess!!<br>\n";
				}
				
				// Change the fetch mode
				$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
				

				if ( $logID=='' )
				{
					// -- Perpare the SQL query string --
					// Select all simular result
					$query = "SELECT * FROM ".$this->_DB_TBL;
					$tail  = '';

					// --- Check the parameter entered, to complete the tail of query ---
					if ( $logCode and $tail)
						$tail .= " AND log_code='".$logCode."'";
					elseif ( $logCode )
							$tail = "log_code='".$logCode."'";

					if ( $logDateTime and $tail)
						$tail .= " AND log_time LIKE '%".$logDateTime."%'";
					elseif ( $logDateTime )
						$tail = "log_time LIKE '%".$logDateTime."%'";

					if ( $logUserID and $tail)
						$tail .= " AND log_user_id='".$logUserID."'";
					elseif ( $logUserID )
						$tail = "log_user_id='".$logUserID."'";
					// --- End of tail of query checking ---


					// If any parameter entered, include the tail part.
					if ( $tail )
						$query .= " WHERE ( $tail ) ";

					// Order by is set, generate the order by query string
					if ( $orderBy )
					{
						$orderArray = explode( ',', $orderBy );
						$orderBy    = '';
						for ( $i=0; $i<count($orderArray); $i++ )
						{
							$orderBy .= ' '.$orderArray[$i].' '.$this->orderMode;
							if ( ($i+1)<count($orderArray) )
								$orderBy .= ',';
						}

						$query .= ' ORDER BY '.$orderBy;
					}
				}
				else
				{
					// -- Perpare the SQL query string --
					// Select only 1 record by the $logID
					$query = "SELECT * FROM ".$this->_DB_TBL." WHERE id='".$logID."'";
				}

				if ( $this->debug )
				{
					// In Debug Mode
					echo 'Query = '.$query."<br>\n";
				}

				// Store the query string into $_LAST_QUERY
				$this->_LAST_QUERY = $query;

				// Clear the array
				$this->readArray = array();

				// Check using SelectLimit or Execute mode
				if ( $this->selectLimit )
				{
					// Checking the selection is the same as before, 
					// if any of the selection condition is changed, 
					// reset the offset pointer.
					if ( $this->_FIND_CODE != $logCode || 
						 $this->_FIND_DATETIME != $logDateTime ||
						 $this->_FIND_USERID != $logUserID )
					{
						// reset the $_selectOffset
						$this->_selectOffset = 0;
						$this->_FIND_CODE     = $logCode;
						$this->_FIND_DATETIME = $logDateTime;
						$this->_FIND_USERID   = $logUserID;
					}
					
					// SelectLimit
					@ $rs = $db->SelectLimit( $query, $this->selectLimit, 
						                      $this->_selectOffset );

					// Change the offset pointer
					$this->_selectOffset += $this->selectLimit;
				}
				else
				{
					// reset the $_selectOffset
					$this->_selectOffset = 0;

					// execute the query
					@ $rs = $db->Execute($query);
				}


				if ( !$rs )
				{
					// Error on select from table
					$this->error = $db->ErrorMsg();

					if ( $this->debug )
					{
						// In Debug Mode
						echo 'Error = '.$this->error."<br>\n";
					}

					return FALSE;						
				}
				else
				{
					// Get total number of record founded
					// *** REMEMBER!! This will not be affected by select limit ***
					@ $rs_temp = $db->Execute($query);
					$this->recordNum = $rs_temp->PO_RecordCount();

					// check the number of result
					$numrows = 0;
					$numrows = $rs->PO_RecordCount($this->_DB_TBL);
					
					if ( !$numrows )
					{
						// Can't fount any simular log event from database
						$this->error = "NO_RECORD";
					}
					else
					{
						for ( $i=0; $i<$numrows; $i++ )
						{
							// Successful retrival the data
							$this->readArray[$i] = $rs->fields;

							$rs->MoveNext();
						}
					}


					// Display debug message
					if ( $this->debug )
					{
						echo 'Total number of record = '.intval($this->recordNum)."<br>\n";
						if ( $this->selectLimit )
							echo 'Select Limit = '.$this->selectLimit."<br>\n";
						echo 'Fetch array = ';
						
						if ( ( $this->recordNum > 1 ) && 
							 (($this->_selectOffset - $this->selectLimit) < intval($this->recordNum) ) )
						{
							echo ($this->_selectOffset - $this->selectLimit).' to ';
							if ( ( $this->selectLimit ) &&
								 (($this->_selectOffset - 1) <= intval($this->recordNum) ) )
								echo ($this->_selectOffset - 1)."<br>\n";
							else
								echo $this->recordNum."<br>\n";
						}
						else
							echo "0<br>\n";
						
						for ( $i=0; $i<count($this->readArray); $i++ )
						{
							$fetchArray = $this->readArray[$i];
							$fetchArrayFlip = array_keys($fetchArray);
							foreach ( $fetchArrayFlip as $arrayName )
							{
								$sess_var_name  = 'TEMP_'.strtoupper( $arrayName );
								$$sess_var_name = noDoubleSpace( $fetchArray[$arrayName] );
							}
							$count = $this->_selectOffset - $this->selectLimit + $i;

							echo "<br>\n";
							echo 'array['.($count)."]['id'] = ".
								 $TEMP_ID.' : ('.strlen($TEMP_ID).')'."<br>\n";
							echo 'array['.($count)."]['log_code'] = ".
								 $TEMP_LOG_CODE.' : ('.strlen($TEMP_LOG_CODE).')'."<br>\n";
							echo 'array['.($count)."]['log_time'] = ".
								 $TEMP_LOG_TIME.' : ('.strlen($TEMP_LOG_TIME).')'."<br>\n";
							echo 'array['.($count)."]['log_event'] = ".
								 $TEMP_LOG_EVENT.' : ('.strlen($TEMP_LOG_EVENT).')'."<br>\n";
							echo 'array['.($count)."]['log_user_id'] = ".
								 $TEMP_LOG_USER_ID.' : ('.strlen($TEMP_LOG_USER_ID).')'."<br>\n";
						}
						echo "<hr>\n";
					}


					if ( !$numrows )
						return FALSE;
					else
						return TRUE;				
				}
			}
		}
		// -- End of private function :: _getLog(..) --

	}
	// -- End of class :: Log --
?>