<?php
	// File        : /include/lib-mydb.inc.php
	//
	// Description : This class contain the method is used to access the database


	// Include the configuration file.
	include_once ( $CONF_CONFIG_PATH.'/config.inc.php' );
	
	// Include the main include file for running all routine.
	include_once ( $CONF_INCLUDE_PATH.'/library.inc.php' );

	// Include the ADOdb Class
	include_once ( $CONF_INCLUDE_PATH.'/adodb/adodb.inc.php');


	// --------------------------------------------------------------------------------------- //
	// Class :: myDB                                                                           //
	// --------------------------------------------------------------------------------------- //
	// Description  : This class contain all the method is used to help the programmer access  //
	//                the database without mine the changes of database system. The variable,  //
	//                function and usage is listed on below.                                   //
	// --------------------------------------------------------------------------------------- //
	// Variable     :   $debug=0;                   // Change to 1 to enable debug mode        //
	// (Public)         $error;                     // Error message                           //
	//                  $readArray=array();         // All simular record in admin table       //
	//                  $selectLimit=0;             // Selecting limit (only work for find())  //
	//                                              // -- 0 mean no selection limit            //
	//                  $recordNum;                 // Total number of the record are founded. //
	//                  $searchRegular='AND';       // Search regular accept ('AND','OR only)  //
	//                  $searchMode='LIKE';         // Search mode accept ('LIKE', '=' only)   //
	//                  $orderBy='id,adm_login_id'; // Order by which column                   //
	//                  $orderMode='DESC';          // DESC = Descending Order                 //
	//                                              // ASC  = Ascending Order                  //
	//                  $keyField='id';             // The name of the key field in a table    //
	//                                                                                         //
	// (Private)        $_DB_TBL='';                // Default table used                      //
	//                  $_DB_TYPE;                  // database information                    //
	//                  $_DB_ADDR;                                                             //
	//                  $_DB_PORT;                                                             //
	//                  $_DB_NAME;                                                             //
	//                  $_DB_USER;                                                             //
	//                  $_DB_PWD;                                                              //
	//                  $_LAST_QUERY;               // The last query string                   //
	//                  $_LAST_RECORD=array();      // Temp. var store the last finded record  //
	//                  $_selectOffset=0;           // Offset (only work for find())           //
	// --------------------------------------------------------------------------------------- //
	// Method       :   changeTable( $table='' );                                              //
	// (Public)         error();                                                               //
	//                  getRecord( $inData='' );                                               //
	//                  update( $inData=array() );                                             //
	//                  insert( $inData=array() );                                             //
	//                  delRecord( $inData='' );                                               //
	//                  query( $query );                                                       //
	//                                                                                         //
	// (Private)        _Query( $mode=0, $inData='' );                                         //
	// --------------------------------------------------------------------------------------- //
	// Examples     :  (Select all record in a table, without limit the result on each time)   //
	//                    $mydb = new myDB('TABLE');                                           //
	//                    $mydb->selectLimit=0;                                                //
	//                    $result = $mydb->getRecord();                                        //
	//                                                                                         //
	//                 (Select all record in a table, with a limit of 5 for each time. If you  //
	//                  have total 14 results, you must do getRecord() 3 times to get all the  //
	//                  result.) * You can check the total result found by : $mydb->recordNum *//
	//                    $mydb->selectLimit=5;                                                //
	//                    $result = $mydb->getRecord();    X  3 Times                          //
	//                                                                                         //
	//                 (Select the record with the conditions, the following command will the  //
	//                  same of this query: [ SELECT * FROM myTABLE WHERE name LIEK '%Test%'   //
	//                  AND favor LIKE '%apple%' ORDER BY id, name ASC ] )                     //
	//                    $mydb = new myDB('myTABLE');                                         //
	//                    $mydb->searchRegular = 'AND';                                        //
	//                    $mydb->searchMode    = 'LIKE';                                       //
	//                    $mydb->orderMode     = 'ASC';                                        //
	//                    $mydb->orderBy       = 'id, name';   <= Assume the table have the    //
	//                    $array['name']       = 'Test';          field (id, name)             //
	//                    $array['favor']      = 'apple';                                      //
	//                    $result              = $mydb->getRecord ( $array );                  //
	//                                                                                         //
	//                 (Update a ROW in the table, the following program will provide the same //
	//                  as the query :                                                         //
	//                       [ UPDATE myTABLE SET name='Test', favot='apple' WHERE id=10 ]     //
	//                  Remember you must provide the name of KEY FIELD inorder to update the  //
	//                  unique row in the table.)                                              //
	//                    $mydb = new myDB('myTABLE');                                         //
	//                    $mydb->keyField      = 'id';                                         //
	//                    $array['id']         = '10';                                         //
	//                    $array['name']       = 'Test';                                       //
	//                    $array['favor']      = 'apple';                                      //
	//                    $mydb->update ( $array );                                            //
	//                                                                                         //
	//                 (Insert a ROW into the table, the operation is simulator to update. But //
	//                  you must remind that this insert will automatically increase the KEY   //
	//                  field by 1. So you must confirm that the table must have a unique field//
	//                  and the propriety of that field is a numeric value. The query will like//
	//                   [INSERT INTO myTABLE (id, name, favor) VALUES (??, 'Test', 'apple')] )//
	//                    $mydb = new myDB('myTABLE');                                         //
	//                    $mydb->keyField      = 'id';                                         //
	//                    $array['id']         = '10';     <= This field is the key field,     //
	//                    $array['name']       = 'Test';      so it will be ignore and the     //
	//                    $array['favor']      = 'apple';     number will be assign by program //
	//                    $mydb->insert ( $array );                                            //
	//                                                                                         //
	//                 (Delete a ROW in a table. The ROW will be delete is 'id'=25.)           //
	//                    $mydb = new myDB('myTABLE');                                         //
	//                    $mydb->keyField  = 'id';                                             //
	//                    $mydb->delRecord( 25 );                                              //
	//                                                                                         //
	//                 (Execute a user defined SQL query command)                              //
	//                    $queryString='CREATE TABLE XYZ (id INT,name CHAR(20),favor CHAR(10))'//
	//                    $mydb->query( $queryString );                                        //	
	// --------------------------------------------------------------------------------------- //
	class myDB
	{
		// -- Variable declaration --
		var		$debug=0;					// Change to 1 to enable debug mode
		var		$error;						// Error message
		var		$readArray=array();			// All simular record in admin table
		var		$selectLimit=0;				// Selecting limit (only work for find())
		                                    // -- 0 mean no selection limit
		var		$recordNum;					// Total number of the record are founded.
		var		$searchRegular='AND';		// Search regular accept ('AND','OR only)
		var		$searchMode='LIKE';			// Search mode accept ('LIKE', '=' only)
		var     $orderBy='id';	            // Order by which column
		var		$orderMode='DESC';			// DESC = Descending Order
											// ASC	= Ascending Order
		var		$keyField='id';				// The name of the key field in a table


		var		$_DB_TBL='';				// Default table used
		var		$_DB_TYPE;					// database information
		var		$_DB_ADDR;
		var		$_DB_PORT;
		var		$_DB_NAME;
		var		$_DB_USER;
		var		$_DB_PWD;
		var		$_LAST_QUERY;				// The last query string
		var		$_LAST_RECORD=array();		// The temp. var store the last finded record
		var		$_selectOffset=0;			// Offset (only work for find())
		


		// ===================================================================
		// == Constructor		myDB::myDB                                  ==
		// ==																==
		// == It will automatic change the table if you provided.           ==
		// ==																==
		// == Usage : $mydb = new myDB( $TABLE );							==
		// == (OR)	: $mydb = new myDB();									==
		// ===================================================================
		function myDB( $tableName='' )
		{
			GLOBAL
				$CONF_DATABASE_TYPE,
				$CONF_DATABASE_ADDR,
				$CONF_DATABASE_PORT,
				$CONF_DATABASE_DBNAME,
				$CONF_DATABASE_ID,
				$CONF_DATABASE_PWD;

			// get the database information from configuration file
			$this->_DB_TYPE = $CONF_DATABASE_TYPE;
			$this->_DB_ADDR = $CONF_DATABASE_ADDR;
			$this->_DB_PORT = $CONF_DATABASE_PORT;
			$this->_DB_NAME = $CONF_DATABASE_DBNAME;
			$this->_DB_USER = $CONF_DATABASE_ID;
			$this->_DB_PWD  = $CONF_DATABASE_PWD;
			$this->_DB_TBL  = $tableName;

			// Display debug message
			if ( $this->debug )
			{
				echo 'Class -= Database =- ....<br>';
				echo '_DB_TYPE = '.$this->_DB_TYPE."<br>\n";
				echo '_DB_ADDR = '.$this->_DB_ADDR."<br>\n";
				echo '_DB_PORT = '.$this->_DB_PORT."<br>\n";
				echo '_DB_NAME = '.$this->_DB_NAME."<br>\n";
				echo '_DB_USER = '.$this->_DB_USER."<br>\n";
				echo '_DB_PWD = '.$this->_DB_PWD."<br><br>\n";
			}
		}
		// -- End of constructor --


		// ===================================================================
		// == Class function	myDB::changeTable( $table='' );				==
		// ==																==
		// == Change to another table in this database.                     ==
		// ==																==
		// == Usage : $mydb->changeTable( 'XYZ' );							==
		// ===================================================================
		function changeTable( $table='' )
		{
			// Display debug message
			if ( $this->debug )
				echo 'Change to another table : '.$table.'  (Previous is : '.$this->_DB_TBL." )<br>\n";

			// Put the new table name into class variable
			$this->_DB_TBL  = $table;
		}
		// -- End of class function :: error() --

		
		// ===================================================================
		// == Class function	myDB::error();  					        ==
		// ==																==
		// == Return the error message.                                     ==
		// ==																==
		// == Usage : $error = $myDB->error();	    						==
		// ===================================================================
		function error()
		{
			if ( $this->error )
				// Return the error message
				return $this->error;
			else
				return FALSE;

			// -- Error message --
			//	UNKNOW_TABLE	: The user not provide a table
			//	UNKNOW_ERROR	: can't connect to DB, unknow reason.
			//	NO_RECORD		: None of record founded! (mode=0 only)
			//	CANNOT_SELECT	: can't select the record
			//	CANNOT_UPDATE	: can't update the table
			//	CANNOT_INSERT	: can't insert a new row into table
			//	CANNOT_DELETE	: can't delete a old row from table
			//	CANNOT_EXECUTE	: can't execute the SQL query
		}
		// -- End of class function :: error() --

		
		// ===================================================================
		// == Class function    myDB::getRecord( $inData='' );              ==
		// ==                                                               ==
		// == Case 1 : If the $inData is a number, it will get a unique     ==
		// ==          record from the table and return the result.         ==
		// ==          (i.e. the number is the KEY field of a record)       ==
		// == Case 2 : If the input is NULL, it will return all record in   ==
		// ==          this table.                                          ==
		// == Case 3 : If the input is an array, it will return the record  ==
		// ==          according by the input data $inData. And the result  ==
		// ==          is control by the select method and WHERE condition. ==
		// ==                                                               ==
		// == Usage : Case 1: $mydb->getRecord( 12 );                       ==
		// ==         Case 2: $mydb->getRecord();                           ==
		// ==         Case 3: $mydb->getRecord( array() );                  ==
		// ===================================================================
		function getRecord( $inData='' )
		{
			// The input data in not an array and it is a number,
			// then assume it is an KEY number of a table.
			if ( !is_array($inData) && is_numeric($inData) )
			{
				// Save the current variable
				$_selectOffset       = $this->_selectOffset;
				$_selectLimit        = $this->selectLimit;
				$_searchMode         = $this->searchMode;
				$this->_selectOffset = 0;
				$this->selectLimit   = 1;
				$this->searchMode    = '=';

				// The input $ID is not an array, so assume this is the
				// number of a ROW and assume the table have a field
				// call 'id' (by default)
				@ $this->_Query( 0, array( $this->keyField=>$inData ) );


				// Return the variable to before
				$this->_selectOffset = $_selectOffset;
				$this->selectLimit   = $_selectLimit;
				$this->searchMode    = $_searchMode;
			}
			else
				// The input is an array, so searching with all condition
				// by the $inData
				@ $this->_Query( 0, $inData );

			// return the result
			return $this->readArray;
		}
		// -- End of class function :: read(..) --


		// ===================================================================
		// == Class function    myDB::update( $inData=array() );            ==
		// ==                                                               ==
		// == Update a row of a table. Remember that you should provide an  ==
		// == array stored the data to be update, and the key of the array  ==
		// == should according to the table field name, otherwise you may   ==
		// == getting serious. Ans also, you should update the class        ==
		// == variable $mydb->keyField according to the KEY field of the    ==
		// == table. Because it is used to check which ROW of record to be  ==
		// == update.                                                       ==
		// == For example, if your table have the fields (ID,Name,HOBBY).   ==
		// == And the KEY field is 'ID', you must update the variable       ==
		// == $mydb->keyField='ID' by yourself before do the update.        ==
		// ==                                                               ==
		// == Usage : $mydb->keyField='ID';                                 ==
		// ==         $mydb->update( array() );                             ==
		// ===================================================================
		function update( $inData=array() )
		{
			return $this->_Query( 1, $inData );
		}

		
		// ===================================================================
		// == Class function    myDB::insert( $inData=array() );            ==
		// ==                                                               ==
		// == Insert a new ROW into the table. Please remember that, you    ==
		// == must provide the name of KEY field to class variable          ==
		// == $mydb->keyField. Because the program will automatic increase  ==
		// == by 1 to that field to prevent a duplicate row.                ==
		// ==                                                               ==
		// == Usage : $mydb->keyField='ID';                                 ==
		// ==         $mydb->update( array() );                             ==
		// ===================================================================
		function insert( $inData=array() )
		{
			return $this->_Query( 2, $inData );
		}


		// ===================================================================
		// == Class function    myDB::delRecord( $inData='' );              ==
		// ==                                                               ==
		// == Delete a ROW from a table. The input data should not be an    ==
		// == array, and it is used indicate which row should be delete.    ==
		// == The inData is according by the key field.                     ==
		// == For exmaple, the the $mydb->keyField='id' and inData is a     ==
		// == number of 10. The it will delete the row which id=10.         ==
		// ==                                                               ==
		// == Usage : $mydb->keyField='ID';                                 ==
		// ==         $mydb->update( 10 );                                  ==
		// ===================================================================
		function delRecord( $inData='' )
		{
			if ( !is_array($inData) )
				return $this->_Query( 3, $inData );
			else
			{
				$this->error = 'CANNOT_DELETE';
				
				if ( $this->debug )
					echo "The entered data is an array, so can not delete a row!<br>\n";

				return FALSE;
			}
		}


		// ===================================================================
		// == Class function    myDB::query( $query );                      ==
		// ==                                                               ==
		// == Execulate a query and return TRUE on successful.              ==
		// ==                                                               ==
		// ==                     *** REMEMBER **                           ==
		// == This function will not return the result on execute a query.  ==
		// == For example, you can not get the record of rows after using   ==
		// == this function to execute a SQL command.                       ==
		// == (e.g. 'SELECT * FROM XYZ')                                    ==
		// ==                                                               ==
		// == Usage : $result = $mydb->query( 'DROP TABLE XYZ' );           ==
		// ===================================================================
		function query( $query )
		{
			// Don't allow the query is an array
			if ( !is_array($query) )
			{
				// Call the private function to execulate a query string
				return $this->_Query( 4, $query );
			}
			else
				// The query is an array, return FALSE
				return FALSE;
		}

	
		// ===================================================================
		// == Class private function                                        ==
		// == OESgroup::_Record ( $mode=0, $admRecord=array() )             ==
		// ==                                                               ==
		// == Private function to create/update/remove a record from the    ==
		// == database of administrator account.                            ==
		// ==                                                               ==
		// == Usage : $admAcc->_Record( 0, array() );                       ==
		// ==                                                               ==
		// ==   $mode = 0     ->   Read a record                            ==
		// ==   $mode = 1     ->   Update record                            ==
		// ==   $mode = 2     ->   Add a new record                         ==
		// ==   $mode = 3     ->   Remove record                            ==
		// ==   $mode = 4     ->   Execute a SQL query ($inData is SQL cmd) ==
		// ===================================================================
		function _Query( $mode=0, $inData='' )
		{
			if ( $this->debug )
			{
				// In Debug Mode
				echo '<hr>';
				echo 'The operation mode = '.$mode." (0=Select, 1=Update, 2=New Row, 3=Remove, 4=Execute SQL Command) <br>\n";
				echo "The input data is listed below : <br>\n";

				if ( ($mode<=3) && is_array($inData) )
				foreach ( array_keys($inData) as $rec )
					echo '$inData['.$rec.'] = '.$inData[$rec]."<br>\n";
			}

			$db = ADONewConnection(db2adodb($this->_DB_TYPE));
			if ($this->debug == true)
				$db->debug = 1;
			if ( chkDBIsODBC($this->_DB_TYPE) )
			{
				// The server type is ODBC, remove PORT setting
				// ...................
				// ..................
			}
			else
			{
				// Check the sql server with different port
				if ( $this->_DB_PORT )
					$sqlServer = $this->_DB_ADDR.":".$this->_DB_PORT;
				else
					$sqlServer = $this->_DB_ADDR;
			}


			// If user provide the database name, auto change to that database.
			if ( $this->_DB_NAME )
				$db->database = $this->_DB_NAME;

			if ( $this->debug )
			{
				// In Debug Mode
				echo "<br>\n";
				echo 'Database   = '.$this->_DB_NAME."<br>\n";
				echo 'Table      = '.$this->_DB_TBL."<br>\n";

				if ( !$this->_DB_TBL || $this->_DB_TBL=='' )
				{
					echo "Error : Empty of table!!<br>\n";
					echo "Exit this function....<br>\n";

					// No table provided
					$this->error = "UNKNOW_TABLE";

					return FALSE;
				}

				echo 'SQL Server = '.$sqlServer;
			}


			// Create Connection
			@ $conn = $db->PConnect($sqlServer, $this->_DB_USER, $this->_DB_PWD);

			if ( !$conn )
			{
				// -- Connection Error --
				// If the $db->ErrorMsg() store some error message
				if ( $db->ErrorMsg() )
					$this->error =  $db->ErrorMsg();
				else
					// Check connection
					$this->error = "UNKNOW_ERROR";

				if ( $this->debug )
					// In Debug Mode
					echo $this->error."<br>\n";

				return FALSE;
			}
			else
			{
				if ( $this->debug )
				{
					// In Debug Mode
					echo "Connect Sucess!!<br>\n";
					echo 'get_magic_quotes_gpc() = '.get_magic_quotes_gpc()."<br>\n";
					echo 'get_magic_quotes_runtime() = '.get_magic_quotes_runtime()."<br>\n";
				}
				
				// Check the current magic_quotes setting
				if ( get_magic_quotes_gpc()==0 )
				{
					if ( !is_array($inData) )
						// In the input data is not an array
						$inData = addslashes ( $inData );
					else
					{
						// Addslashes to prevent the special character input
						// (e.g. Chinese Input '\\\'...etc)
						foreach ( array_keys($inData) as $rec )
							$inData[$rec] = addslashes ( $inData[$rec] );
					}

					if ( $this->debug )
					{
						echo "<br>\n";
						echo "The following data is after addslashed to prevent some special characters : <br>\n";
						if ( is_array($inData) )
							foreach ( array_keys($inData) as $rec )
								echo '$inData['.$rec.'] = '.$inData[$rec]."<br>\n";
						else
							echo '$inData = '.$inData."<br>\n";
					}
				}


				// ---------------------------------- //
				// -- Checking the operation mode  -- //
				// ---------------------------------- //
				if ( $mode == 0 )
				{
					// *** Mode 0 ***

					// Perpare the SQL query string
					$query = "SELECT * FROM ".$this->_DB_TBL;

					// Only add the WHERE expression into sql query command,
					// when the programmer provide an array of $inData
					if ( is_array($inData) )
					{
						// Addslashes to prevent the special character input
						// (e.g. Chinese Input '\\\'...etc)
						foreach ( array_keys($inData) as $rec )
							$inData[$rec] = addslashes ( $inData[$rec] );

						if ( strtoupper($this->searchMode) == 'LIKE' )
						{
							$searchMode = 'LIKE';
							$percent = '%';
						}
						elseif ( strtoupper($this->searchMode) == '=' )
						{
							$searchMode = '=';
							$percent = '';
						}

						if ( $this->debug )
						{
							echo 'The search mode $this->searchMode "'.$this->searchMode."\"<br>\n";
							echo 'The search regular $this->searchRegular "'.$this->searchRegular."\"<br>\n";
						}

						// --------------------------------------------------------- //
						// Generating the query that support the searching condition
						// --------------------------------------------------------- //
						$tail  = '';
						$queryTail = '';
						$i=0;

						// Insert the searching condition string in each of entered field
						foreach ( array_keys($inData) as $key )
							$tail[$i++] .= " $key $searchMode '$percent".$inData[$key]."$percent' ";

						if ( count($tail) <= 1 )
							// Only one field need to be search with condition
							$queryTail = $tail[0];
						else
							for ( $i=0; $i<count($tail); $i++ )
								// Don't add 'AND' or 'OR' at the end of that query tail
								if ( ($i+1)<count($tail) )
									$queryTail .= $tail[$i].$this->searchRegular;
								else
									$queryTail .= $tail[$i];
						// --------------------------------------------------------- //


						// If the input data contain
						if ( $queryTail )
							$query .= ' WHERE '.$queryTail;

						// If set the order by, the query result will be sorted by which field.
						if ( $this->orderBy )
							$query .= ' ORDER BY '.$this->orderBy;

						// If set the order mode (i.e. the result is sort by Accending or Descending mode)
						if ( $this->orderMode )
							$query .= ' '.$this->orderMode;

						// Clear the array
						$this->readArray = array();
					}


					// Check using SelectLimit or Execute mode
					if ( $this->selectLimit )
					{
						// Checking the selection is the same as before, 
						// if any of the selection condition is changed, 
						// reset the offset pointer.
						if ( $this->_LAST_RECORD != $inData )
						{
							// reset the $_selectOffset
							$this->_selectOffset = 0;
							$this->recordNum = 0;
							$this->_LAST_RECORD = $inData;
						}
					}
					else
					{
						// reset the $_selectOffset
						$this->_selectOffset = 0;
						$this->recordNum = 0;
					}

					// Change the fetch mode
					$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;

					// *** End of Mode 0 ***
				}
				elseif ( $mode == 1 )
				{
					// --------------------------------------------- //
					// Update the record in a table                  //
					// --------------------------------------------- //

					// Do not allow the input data is not an array
					if ( !is_array($inData) )
						return FALSE;
					else
					{
						// Query an empty record set
						// Assume the 'id' is the key of a table 
						// (by default $this->keyField='id')
						$sql = "SELECT * FROM ".$this->_DB_TBL." WHERE ".$this->keyField." = ".$inData[$this->keyField]; 
						@ $rs = $db->Execute($sql);

						// Get the insert quert string
						@ $query = $db->GetUpdateSQL($rs, $inData);
					}
				}
				elseif ( $mode == 2 )
				{
					// --------------------------------------------- //
					// Insert a new record to table                  //
					// --------------------------------------------- //

					// Do not allow the input data is not an array
					if ( !is_array($inData) )
						return FALSE;
					else
					{
						// Get the next avaliable ID and the current date and time
						// (by default the key field $this->keyField='id')
						 $inData[$this->keyField] = $db->GenID($this->_DB_TBL.'_id_seq');

						// Query an empty record set
						$sql = "SELECT * FROM ".$this->_DB_TBL." WHERE id = -1";
						@ $rs = $db->Execute($sql);

						// Get the insert quert string
						@ $query = $db->GetInsertSQL($rs, $inData, true);
					}
				}
				elseif ( $mode == 3 )
				{
					// --------------------------------------------- //
					// Delete an old record from table               //
					// --------------------------------------------- //
					$query = "DELETE FROM ".$this->_DB_TBL." WHERE ".$this->keyField."='$inData'";
				}
				elseif ( $mode == 4)
				{
					// In the input data '$inData' is not an array, then
					// assume it is the query string
					if ( !is_array($inData) )
						$query = $inData;
				}

				if ( $this->debug )
					// In Debug Mode
					echo 'Query = '.$query."<br>\n";

				// Store the query string into $_LAST_QUERY
				$this->_LAST_QUERY = $query;

				// execute the query
				if ( $mode == 0 && $this->selectLimit )
				{
					// SelectLimit
					@ $rs = $db->SelectLimit( $query, $this->selectLimit, $this->_selectOffset );

					// Increase the offset pointer
					$this->_selectOffset += $this->selectLimit;
				}
				else
					@ $rs = $db->Execute($query);

				
				// Check the execution of query is successful or not
				if ( $rs )
				{
					// *** Execute Successful!! ***

					// Check the operation mode and do the apporate action
					switch ( $mode )
					{
						// Mode 0 is select the record from a given table
						case 0	:	if ( !$this->recordNum )
									{
										// Get total number of record founded
										// *** REMEMBER!! This will not be affected by select limit ***
										@ $rs_temp = $db->Execute($query);
										$this->recordNum = $rs_temp->PO_RecordCount();
									}

									// check the number of result
									$numrows = 0;
									$numrows = $rs->PO_RecordCount($this->_DB_TBL);

									if ( !$numrows )
									{
										// Can't fount any simular log event from database
										$this->error = "NO_RECORD";
									}
									else
									{
										for ( $i=0; $i<$numrows; $i++ )
										{
											// Successful retrival the data
											$this->readArray[$i] = $rs->fields;
											$rs->MoveNext();
										}
									}

									// Display debug message
									if ( $this->debug )
									{
										echo "<br>\n";
										echo 'Total number of record = '.intval($this->recordNum)."<br>\n";
										if ( $this->selectLimit )
											echo 'Select Limit = '.$this->selectLimit."<br>\n";
										echo 'Fetch array = ';
										
										if ( ( $this->recordNum > 1 ) && 
											 (($this->_selectOffset - $this->selectLimit) < intval($this->recordNum) ) )
										{
											echo ($this->_selectOffset - $this->selectLimit);
											if ( $numrows <= 1 )
												echo "<br>\n";
											else
												if ( ( $this->selectLimit ) &&
													 (($this->_selectOffset - 1) <= intval($this->recordNum) ) )
													echo ' to '.($this->_selectOffset - 1)."<br>\n";
												else
													echo ' to '.$this->recordNum."<br>\n";
										}
										else
											echo "0<br>\n";

										echo '$this->_selectOffset = '.$this->_selectOffset ."<br>\n";
										echo '$this->selectLimit = '.$this->selectLimit."<br>\n";

										for ( $i=0; $i<$numrows; $i++ )
										{
											echo "<br>\n";
											$count = $this->_selectOffset - $this->selectLimit + $i;
											foreach ( array_keys($this->readArray[$i]) as $key )
											{
												$tempVar = noDoubleSpace( $this->readArray[$i][$key] );
												echo "array[$count]['".$key."'] = $tempVar (".strlen($tempVar).")<br>\n";
											}
										}
										echo "<hr>\n";
									}

									if ( !$numrows )
										// None of record found!!
										return FALSE;
									else
										return TRUE;

									break;


						// After execute the query
						case 1	:
						case 2	:
						case 3	: 
						case 4	:	if ( $this->debug )
										echo "Execute the query : Success!!<br>\n";
									
									// Retun TRUE on success
									return TRUE;

									break;
					}
				}
				// *** Execute Fail!! ***
				else
				{
					// Execulate the query have any error
					// If the $db->ErrorMsg() store some error message, save this error
					if ( $db->ErrorMsg() )
						$this->error =  $db->ErrorMsg();
					else
					{
						switch ( $mode )
						{
							case 0	:	// Can't select the result
										$this->error = "CANNOT_SELECT";
										break;
							case 1	:	// Can't update the record
										$this->error = "CANNOT_UPDATE";
										break;
							case 2	:	// Can't insert a new row to table
										$this->error = "CANNOT_INSERT";
										break;
							case 3	:	// Can't delete the row of record
										$this->error = "CANNOT_DELETE";
										break;
							case 4	:	// Can't execute the SQL command
										$this->error = "CANNOT_EXECUTE";
										break;
						}
					}

					if ( $this->debug )
						// In Debug Mode
						echo 'Error = '.$this->error."<br><hr>\n";

					return FALSE;						
				}

			}
		}
		// -- End of private function :: _Record(..) --	
	
	}
	// -- End of class :: myDB --
?>