<?php
	// File        : /include/lib-template-mod.inc.php
	//
	// Description : This library contain all the class or function for themes.


	// Include the configuration file.
	include_once ( $CONF_CONFIG_PATH.'/config.inc.php' );
	
	// Define the default path of 'file.php', this only be use for the image tag.
	define ("DEFAULT_file_php"			, $CONF_ADMIN_URL.'/file.php'		,TRUE);
	

	// ------------------------------------------------------------------------------------ //
	// Class :: myModTmp                                                                    //
	// ------------------------------------------------------------------------------------ //
	// Description  : This class contain the method provided for the modules developer more //
	//                easier to create their template for the modules.                      //
	// ------------------------------------------------------------------------------------ //
	// Variable     :   $debug=0;                   // Set to 1 to enable debug mode        //
	// (Public)         $template='';               // the modules's template file          //
	//                  $lang='';                   // the language name (e.g 'en')         //
	//                  $selfPath=0;                // 1=loading the template from any where//
	//                                              // (within the OES)                     //
	//                                              // 0=only load the template for modules //
	//                  $isAdmin=0;                 // 1=loading the admin modules template //
	//                                              // 0=loading the user modules template  //
	//                                                                                      //
	// (Private)        $_result='';                // the final result                     //
	//                  $_langPath='';              // the language file path               //
	//                  $_category='';              // which category does this modules in  //
	//                  $_tempResult=array();       // the temporary result                 //
	//                                              // (array of repeat ROWs)               //
	//                  $_modules='';               // default to find the current          //
	//                                              // modules is using                     //
	// ------------------------------------------------------------------------------------ //
	// Method       :   myModTmp( $template='', $lang = 'en' );                             //
	// (Public)         getResult();                                                        //
	//                  getLangVAR( $langStr );                                             //
	//                  getTemplate();                                                      //
	//                  assign( $pattern='', $replace='' );                                 //
	//                  repeatRow( $rowname );                                              //
	//                  removeRow( $rowName );                                              //
	// ------------------------------------------------------------------------------------ //
	// Examples     : (Create a myModTmp object, and get the template for 'abc.template'.   //
	//                 which is the administrator modules template and the language is 'EN' //
	//                 (i.e. English), the base path of the template is :                   //
	//                 "$CONF_ADMIN_MODULES_PATH.'/'.$this->_category.'/'.$this->_modules"  //
	//                 and then get the final result to the screen.)                        //
	//                  $myMod = new myModTmp( 'abc.template', 'en' );                      //
	//                  $myMod->debug=0;                                                    //
	//                  $myMod->isAdmin=true;                                               //
	//                  $myMod->getTemplate();                                              //
	//                  echo $myMod->getResult();                                           //
	//                                                                                      //
	//                (Dynamic Assign a value to the template, assume the template have a   //
	//                 dynamic variable {DYNAMIC_VAR})                                      //
	//                  $myMod = new myModTmp( 'abc.template', 'en' );                      //
	//                  $myMod->debug=0;                                                    //
	//                  $myMod->getTemplate();                                              //
	//                  $myMod->assign ( 'DYNAMIC_VAR', 'My Dinamic assignment.!!' );       //
	//                  echo $myMod->getResult();                                           //
	//                                                                                      //
	//                (Repeat a row with a dynamic assignment, assume the ROW name is 'ABC')//
	//                  $myMod->getTemplate();                                              //
	//                  $myMod->assign ( 'DYNAMIC_VAR', 'My Dinamic assignment 1.!!' );     //
	//                  $myMod->repeatRow( 'ABC' );                                         //
	//                  $myMod->assign ( 'DYNAMIC_VAR', 'My Dinamic assignment 2.!!' );     //
	//                  echo $myMod->getResult();                                           //
	//                                                                                      //
	//                (Doubly call a repeat row)                                            //
	//                  for ( $i=0; $i<10; $i++ ) {                                         //
	//                     for ( $j=0; $j<5; $j++ ) {                                       //
	//                        $myMod->assign ( 'DYNAMIC_VAR', $i );                         //
	//                        $myMod->repeatRow( 'inRow' );                                 //
	//                     }                                                                //
	//                     $myMod->removeRow('inRow');                                      //
	//                     $myMod->repeatRow( 'BIG_ROW' );                                  //
	//                  }                                                                   //
	//                  echo $myMod->getResult();                                           //
	// ------------------------------------------------------------------------------------ //
	class myModTmp
	{
		var		$debug=0;							// Set to 1 to enable debug mode
		var		$template='';						// the modules's template file
		var		$lang='';							// the language name (e.g 'en')
		var		$selfPath=0;						// 1=loading the template from any where
													//   (within the OES)
													// 0=only load the template for modules
		var		$isAdmin=0;						    // 1=loading the admin modules template
													// 0=loading the user modules template (default)

		var		$_result='';						// the final result
		var		$_langPath='';						// the language file path
		var		$_category='';						// which category does this modules in
		var		$_tempResult=array();				// the temporary result (array of repeat ROWs)
		var		$_modules='';						// default to find the current
													// modules is using
		

		// ===================================================================
		// == Class function    myModTmp::myModTmp( $theme = '',            ==
		// ==                                       $lang = '' );           ==
		// ==                                                               ==
		// == Allow programmer to select which template file and language   ==
		// == for that modules.                                             ==
		// ==                                                               ==
		// == Usage : $myMod = new myModTmp( 'my.template', 'en' );         ==
		// ===================================================================
		function myModTmp( $template='', $lang = 'en' )
		{
			$this->template		= $template;
			$this->lang			= $lang;

			$this->_category	= $GLOBALS["category"];
			$this->_modules		= $GLOBALS["mod"];		// assume the current modules
														// is store in $mod
		}
		// -- End of constructor --


		// ===================================================================
		// == Class function    myModTmp::getResult();                      ==
		// ==                                                               ==
		// == Return the final result. The result including the template    ==
		// == layout, replace with the sutiable language string, and the    ==
		// == dynamic data.                                                 ==
		// ==                                                               ==
		// == Usage : Display the result in HTML form.                      ==
		// ==         echo $myMod->getResult();                             ==
		// ===================================================================
		function getResult()
		{
			// Show the HTML code when in debug mode
			if ( $this->debug )
				echo highlight_string($this->_result, TRUE);
				

			// Return the final result with repleated ROW
			return $this->_result;
		}
		// -- End of class function :: getResult(..) --


		// ===================================================================
		// == Class function    myModTmp::getLangVAR();                     ==
		// ==                                                               ==
		// == This function can get the variable from the language file.    ==
		// == It is good for the programmer to get a TEXT from different set==
		// == of language file. This function will return the value from the==
		// == apporate language file.                                       ==
		// == By remember the usage must like the following example.        ==
		// ==                                                               ==
		// == Usage : echo $myMod->getLangVAR('LANG=myLangVar');            ==
		// ===================================================================
		function getLangVAR( $langStr )
		{
			if ( 'LANG=' == substr( $langStr, 0, 5) )
			{
				// ------------------------------------------------------------------- //
				// Include the modules language file
				// ------------------------------------------------------------------- //
				GLOBAL	$MODULES,
						$CONF_ADMIN_MODULES_PATH,
						$CONF_MODULES_PATH;

				// If is admin, goto admin modules to get the template
				if ( $this->isAdmin )
					$basePath = $CONF_ADMIN_MODULES_PATH.'/'.$this->_category.'/'.$this->_modules;
				else
					$basePath = $CONF_MODULES_PATH.'/'.$this->_category.'/'.$this->_modules;

				// Stop when the user selected language file readed
				if ( @!include( $this->_langPath = $basePath.$MODULES["LANG_DIR"].'/'.$this->lang.'.lang.php' ) ) 
				{
					if ( $this->debug )
						// Display the can't open language file error message when debug mode is ON
						echo "Can not open the language file '".$this->_langPath."'.<br>\n";

					return FALSE;
				}
				// ------------------------------------------------------------------- //

				$langVar = explode( '=', $langStr );
				return $MODLANG[$langVar[1]];
			}
			else
				return FALSE;
		}
		// -- End of class function :: getLangVAR(..) --


		// ===================================================================
		// == Class function    myModTmp::getTemplate();                    ==
		// ==                                                               ==
		// == Ge the template file and mix with the language file and then  ==
		// == output the result on HTML format.                             ==
		// ==                                                               ==
		// == Usage : $myMod->getTemplate();                                ==
		// ===================================================================
		function getTemplate()
		{
			GLOBAL
				$CONF_ADMIN_MODULES_PATH,
				$CONF_MODULES_PATH;

			// Clear the buffer when getting another template
			$this->_result='';

		
			// Remove all '..' and '../' in the variable file
			// because of security reason
			$this->template = str_replace( '../', '', $this->template );
			$this->template = str_replace( '..', '', $this->template );

			
			// If set the selfPath, load any template from any where
			// but not within the OES system
			if ( $this->selfPath )
				$filename = '/'.$this->template;
			else
			{
				// If is admin, goto admin modules to get the template
				if ( $this->isAdmin )
					$basePath = $CONF_ADMIN_MODULES_PATH.'/'.$this->_category.'/'.$this->_modules;
				else
					$basePath = $CONF_MODULES_PATH.'/'.$this->_category.'/'.$this->_modules;
				$filename = $basePath.'/'.$this->template;
			}


			if ( $this->debug )
				echo "Opening file '$filename' ... <br>\n";


			// Fail if the template file is not found
			if ( !@is_file( $filename ) )
			{
				if ( $this->debug )
					// Debug mode is ON
					// Display can't open the template file message
					echo "Error!! Can not open the file '".$filename." on line 154'. <br>\n";
			}
			else
			{
				// ------------------------------------------------------------------- //
				// Read the content of template file
				// ------------------------------------------------------------------- //
				$fp = @fopen( $filename, 'r' );
				$templateContent = fread ( $fp, filesize ($filename) );
				fclose ($fp);
				// ------------------------------------------------------------------- //


				// Do it if not set the selfPath
				// (i.e. the template is under the modules, and then do the language replacement)
				if ( !$this->selfPath )
				{
					// ------------------------------------------------------------------- //
					// Replace the file link in the template file.
					// Use the php program 'file.php' to get the content of the file file
					// (e.g. <IMG SRC="xx">, <SCRIPT... SRC=""></SCRIPT>, <..BACKGROUND=""> )
					// *** Increase the security ***
					// ------------------------------------------------------------------- //
					$templateArray = explode( "\n", $templateContent );
					$rownum = 0;
					$rowName = array();
					for ( $i=0; $i<count($templateArray); $i++ )
					{
						if ( eregi( 'src="(.*)"', $templateArray[$i], $filePath) ||
							 eregi( 'background="(.*)"', $templateArray[$i], $filePath) )
						{
							// Split the string by '"'
							$filePath = explode( '"', $filePath[1] );

							// Stop if the path is NULL
							if ( !$filePath[0] )
								break;

							// Check the file type
							$strLength = strlen( $filePath[0] );
							if ( 0==strcmp( strtoupper('.js'), strtoupper(substr($filePath[0], $strLength - 3, $strLength))) )
								// The file link is javascript, so the path will different
								$fileType = 'javascript';
							else
								// no default file type
								$fileType = '';

							if ( $fileType )
								// If the file link is set
								$fileLink = $filePath[0];
							else
								// No file link is set, link to default template directory
								$fileLink = 'modules/'.$this->_category.'/'.$this->_modules.'/'.$filePath[0];

							// Replace each line one by one if the line content the file link ('src=', or 'background=')
							if ( $this->speedLimit )
								// Have set the download speed limit
								@ $templateArray[$i] = str_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&file='.$fileLink, $templateArray[$i] );
							else
								// No download speed limit
								@ $templateArray[$i] = str_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&nolimit=&file='.$fileLink, $templateArray[$i] );
						}

						// Get the name of each 'ROW' block, if existed.
						if ( eregi( '<!-- \[ROW NAME="(.*)"\] -->', $templateArray[$i], $rowname) )
						{
							$rowname            = explode( '"', $rowname[1] );
							$rowName[$rownum++] = $rowname[0];
						}
					}
					// Join the result together
					$templateContent = implode ( "\n", $templateArray );

					// Remove the template array from memory
					unset ($templateArray);
					// -------------------- End of replace image link --------------------- //


					// ------------------------------------------------------------------- //
					// Include the modules language file
					// ------------------------------------------------------------------- //
					GLOBAL $MODULES;

					// Stop when the user selected language file readed
					if ( !@include( $this->_langPath = $basePath.$MODULES["LANG_DIR"].'/'.$this->lang.'.lang.php' ) ) 
					{
						if ( $this->debug )
							// Display the can't open language file error message when debug mode is ON
							echo "Can not open the language file '".$this->_langPath."'.<br>\n";
					}
					// ------------------------------------------------------------------- //


					// ------------------------------------------------------------------- //
					// Find the key of string from the language file
					// ------------------------------------------------------------------- //
					if ( !is_array($MODLANG) )
						echo "Can not replace with language patten, please check your language file!<br>\n";
					else
						@$langString = array_keys($MODLANG);
					for ( $i=0; $i<count($langString); $i++ )
					{
						@ $templateContent = str_replace( '['.$langString[$i].']', $MODLANG[$langString[$i]], $templateContent );
					}
					// ------------------------------------------------------------------- //


					// Store all block of 'ROW' into class variable
					foreach ($rowName as $rowname)
					{
						eregi( '<!-- \[ROW NAME="'.$rowname.'"\] -->(.*)<!-- \[ENDROW NAME="'.$rowname.'"\] -->', $templateContent, $block );
						$block = str_replace( '<!-- [ROW NAME="'.$rowname.'"] -->', '', $block[0]);
						$block = str_replace( '<!-- [ENDROW NAME="'.$rowname.'"] -->', '', $block);

						$this->_tempResult[$rowname] = $block;
					}
				}


				// Return the temporary result
				$this->_result = $templateContent;


				// Show the HTML code when in debug mode
				if ( $this->debug )
				{
					echo "<br>\n";
					echo "DEBUG : Showing the source code of temnplate file. (finish the replacement of language)<br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
					echo " Template file : '$filename' <br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
					echo " Dumping the source code....... <br>\n";
					echo highlight_string($this->_result, TRUE);
					echo "<br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
					echo " End of dumping the template source code! <br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
				}
			}
		}
		// -- End of class function :: getTemplate(..) --


		// ===================================================================
		// == Class function   myModTmp::assign( $pattern='', $replace='' );==
		// ==                                                               ==
		// == Replace the patten string with the dynamic conten. This is    ==
		// == very important to replace the dynamic content into the        ==
		// == template layout.                                              ==
		// ==                                                               ==
		// == Usage : $myMod->assign( 'DYNAMIC_VAR', 'some data' );         ==
		// ===================================================================
		function assign( $pattern='', $replace='' )
		{
			// Display the debug content
			if ( $this->debug )
			{
				echo "<br>\n";
				echo "DEBUG : assign ( '$pattern', '$replace' );<br>\n";
				echo "DEBUG : '{$pattern}' => '$replace'<br>\n";
			}

			// -------------------------------------------------------------------- //
			// Do the replacement if the input '$replace' is a refer as a file link.
			// (e.g. <IMG SRC="image.jpg" .....> )
			// -------------------------------------------------------------------- //
			if ( eregi( 'src="(.*)"', $replace, $filePath) ||
				 eregi( 'background="(.*)"', $replace, $filePath) )
			{
				// Split the string by '"'
				$filePath = explode( '"', $filePath[1] );

				// Stop if the path is NULL
				if ( !$filePath[0] )
					break;

				// Remove all '..' and '../' in the variable file
				// because of security reason
				$filePath[0] = str_replace( '../', '', $filePath[0] );
				$filePath[0] = str_replace( '..', '', $filePath[0] );


				// Check the file type
				$strLength = strlen( $filePath[0] );
				if ( 0==strcmp( strtoupper('.js'), strtoupper(substr($filePath[0], $strLength - 3, $strLength))) )
					// The file link is javascript, so the path will different
					$fileType = 'javascript';
				else
					// no default file type
					$fileType = '';

				if ( $fileType )
					// If the file link is set
					$fileLink = $filePath[0];
				else
					// No file link is set, link to default template directory
					$fileLink = 'modules/'.$this->_category.'/'.$this->_modules.'/'.$filePath[0];

				// Replace each line one by one if the line content the file link ('src=', or 'background=')
				if ( $this->speedLimit )
					// Have set the download speed limit
					@ $replace = str_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&file='.$fileLink, $replace );
				else
					// No download speed limit
					@ $replace = str_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&nolimit=&file='.$fileLink, $replace );
			}
			// -------------------- End of replace image link --------------------- //


			// Check the dynamic assign is it a special data
			// If the replace string is the form of 'LANG=XXXX', this template
			// system will try to retrival the string from the language file
			// which the text is named by 'XXXX'.
			$langText = explode( 'LANG=', $replace);
			if (( $langText[1] ) && ( @include( $this->_langPath ) ) )
				// Special replacement from Language file
				$this->_result = str_replace( '{'.$pattern.'}', $MODLANG[$langText[1]], $this->_result );
			elseif ( 'IMG=' == substr( $replace, 0, 4) )
			{
				// The replace string is a images
				$imgLink = explode( 'IMG=', $replace );

				// No file link is set, link to default template directory
				$imgLink = 'modules/'.$this->_category.'/'.$this->_modules.'/'.$imgLink[1];

				// Replace each line one by one if the line content the file link ('src=', or 'background=')
				if ( $this->speedLimit )
					// Have set the download speed limit
					@ $this->_result = str_replace( '{'.$pattern.'}', DEFAULT_file_php.'?&mode=load&file='.$imgLink, $this->_result );
				else
					// No download speed limit
					@ $this->_result = str_replace( '{'.$pattern.'}', DEFAULT_file_php.'?&mode=load&nolimit=&file='.$imgLink, $this->_result );
			}
			else
				// Replace the pattern with the dynamic data.
				$this->_result = str_replace( '{'.$pattern.'}', $replace, $this->_result );
		}
		// -- End of class function :: assign(..) --


	
		// ===================================================================
		// == Class function    myModTmp::repeatRow( $rowname );            ==
		// ==                                                               ==
		// == Place an un-assigned block of row into appropriate place of   ==
		// == template. (i.e. empty row ) The following is the structure of ==
		// == a row.                                                        ==
		// ==                                                               ==
		// ==   <!-- [ROW NAME="inRow"] -->                                 ==
		// ==         Content of the ROW ..............                     ==
		// ==   <!-- [ENDROW NAME="inRow"] -->                              ==
		// ==                                                               ==
		// == Usage : $myMod->repeatRow( 'ROW1' );                          ==
		// ===================================================================
		function repeatRow( $rowname )
		{
			// Add an empty block at the end of the current block of element
			$this->_result = str_replace( '<!-- [ENDROW NAME="'.$rowname.'"] -->', 
 				                           $this->_tempResult[$rowname].'<!-- [ENDROW NAME="'.$rowname.'"] -->',
				                           $this->_result );
		}
		// -- End of class function :: repeatRow(..) --


		// ===================================================================
		// == Class function    myModTmp::removeRow( $rowName );            ==
		// ==                                                               ==
		// == Remove the repeat row in the final result part. This is very  ==
		// == useful for the advance template design. For example if you    ==
		// == want to doubly repeat the row two times. Then you must use    ==
		// == this function to pervent the wrong repeacement.               ==
		// ==                                                               ==
		// ==   <!-- [ROW NAME="BIG_ROW"] -->                               ==
		// ==   {NAME}                                                      ==
		// ==	    <!-- [ROW NAME="inRow"] -->                             ==
		// ==	    {INFORMATION}                                           ==
		// ==	    <!-- [ENDROW NAME="inRow"] -->                          ==
		// ==   <!-- [ENDROW NAME="BIG_ROW"] -->                            ==
		// ==                                                               ==
		// == Usage : $myMod->removeRow('inRow');                           ==
		// ==         $myMod->repeatRow( 'BIG_ROW' );                       ==
		// ==         $myMod->repeatRow( 'inRow' );                         ==
		// ===================================================================
		function removeRow( $rowName )
		{
			$this->_result = str_replace ( '<!-- [ROW NAME="'.$rowName.'"] -->', '', $this->_result );
			$this->_result = str_replace ( '<!-- [ENDROW NAME="'.$rowName.'"] -->', '', $this->_result );
		}
		// -- End of class function :: repeatRow(..) --
	}
	// -- End of class :: myModTmp --
?>
