<?php
	// File        : /include/lib-themes.inc.php
	//
	// Description : This library contain all the class or function for themes.


	// Include the configuration file.
	include_once ( $CONF_CONFIG_PATH.'/config.inc.php' );
	

	// Define the default path of 'file.php', this only be use for the image tag.
	define ("DEFAULT_file_php"			, './file.php'		,TRUE);
	

	// ------------------------------------------------------------------------------------ //
	// Class :: myTheme                                                                     //
	// ------------------------------------------------------------------------------------ //
	// Description  : This class contain the method to access themes and template files.    //
	//                This is vary important classes in OES. Because all the HTML output    //
	//                will related on this class. Without the help of this class, the OES   //
	//                can't display anything on the client browser.                         //
	// ------------------------------------------------------------------------------------ //
	// Variable     :   $debug=0;                   // Set to 1 to enable debug mode        //
	//                  $themeName='OES.default';   // The current themes is using          //
	//                  $themeLang='en';            // The default output language          //
	//                  $speedLimit=0;              // 1=limit the download speed           //
	//                  $isAdmin=0;                 // 1=loading the admin themes           //
	//                                              // 0=loading the user themes            //
	//                                                                                      //
	//                  $_result='';                // the final result                     //
	//                  $_tempResult=array();       // the temporary result                 //
	//                                              // (array of repeat ROWs)               //
	//                  $_langPath='';              // the language file path               //
	// ------------------------------------------------------------------------------------ //
	// Method       :   myFile( $inFileName = '' );                                         //
	// (Public)         setFile( $inFileName = '' );                                        //
	//                  getFile();                                                          //
	//                  loadFile();                                                         //
	//                  getHeader();                                                        //
	//                                                                                      //
	// (Private)    :   _checkTheme();                                                      //
	// ------------------------------------------------------------------------------------ //
	// Examples     : (Create a myfile object)                                              //
	//                  $myfile     = new myFile();                                         //
	//                                                                                      //
	// ------------------------------------------------------------------------------------ //
	class myTheme
	{
		var		$debug=0;							// Set to 1 to enable debug mode
		var		$themeName='OES.default';			// The current themes is using
		var		$themeLang='en';					// The default output language
		var		$speedLimit=0;						// 1=limit the download speed
		var		$isAdmin=0;						    // 1=loading the admin themes
													// 0=loading the user themes
		
		var		$_result='';						// the final result
		var		$_tempResult=array();				// the temporary result (array of repeat ROWs)
		var		$_langPath='';						// the language file path

		

		// ===================================================================
		// == Class function    myTheme::myThemes( $theme = '',             ==
		// ==                                      $lang = '' );            ==
		// ==                                                               ==
		// == If not specific the name of Theme or the language will be use,==
		// == then will use the default theme 'OES.default' and use english ==
		// == to display the content. (i.e. LANG='en')                      ==
		// ==                                                               ==
		// == Usage : $myfile = new myFile();                               ==
		// == (OR)	: $myfile = new myThemes( 'OES.default', 'en' );        ==
		// ===================================================================
		function myTheme( $theme = 'OES.default', $lang = 'en' )
		{
			$this->themeName = $theme;
			$this->themeLang = $lang;

			// If the themes is not enable for use, use the system default themes
			if ( !$this->_checkTheme() )
				$this->themeName = 'OES.default';
		}
		// -- End of constructor --

		

		// ===================================================================
		// == Class function    myTheme::setTheme( $theme = '',             ==
		// ==                                      $lang  = '' );           ==
		// ==                                                               ==
		// == This is an optional function. Only will be use when the       ==
		// == programmer want to change the themes or language set on       ==
		// == run-time.                                                     ==
		// == If not specific the name of Theme or the language will be use,==
		// == then will use the default theme 'OES.default' and use english ==
		// == to display the content. (i.e. LANG='en')                      ==
		// ==                                                               ==
		// == Set the theme will be use.                                    ==
		// == Usage : $myfile->setTheme( 'OES.default', 'en' );             ==
		// ===================================================================
		function setTheme( $theme = 'OES.default', $lang = 'en' )
		{
			$this->themeName = $theme;
			$this->themeLang = $lang;

			// If the themes is not enable for use, use the system default themes
			if ( !$this->_checkTheme() )
				$this->themeName = 'OES.default';
		}
		// -- End of class function :: setTheme(..) --



		// ===================================================================
		// == Class function    myTheme::getResult();                       ==
		// ==                                                               ==
		// == Return the final result. The result including the template    ==
		// == layout, replace with the sutiable language string, and the    ==
		// == dynamic data.                                                 ==
		// ==                                                               ==
		// == Usage : Display the result in HTML form.                      ==
		// ==         echo $myfile->getResult();                            ==
		// ===================================================================
		function getResult()
		{
			// Show the HTML code when in debug mode
			if ( $this->debug )
				highlight_string($this->_result, TRUE);
				

			// Return the final result with repleated ROW
			return $this->_result;
		}
		// -- End of class function :: getResult(..) --



		// ===================================================================
		// == Class function    myTheme::getTemplate( $file = '' );         ==
		// ==                                                               ==
		// == Ge the template file and mix with the language file and then  ==
		// == output the result on HTML format.                             ==
		// ==                                                               ==
		// == Usage : $myfile->getTemplate( 'login.theme.php' );            ==
		// ===================================================================
		function getTemplate( $file = '' )
		{
			GLOBAL
				$CONF_ADMIN_THEMES_PATH,
				$CONF_THEMES_PATH,
				$THEMES_TEMPLATE_DIR,
				$THEMES_LANG_DIR;

			// Clear the buffer when getting another template
			$this->_result='';
			
			// Remove all '..' and '../' in the variable file
			// because of security reason
			$file = str_replace( '../', '', $file );
			$file = str_replace( '..', '', $file );


			// Find the base path of the template file
			if ( $this->isAdmin )
				$basePath = $CONF_ADMIN_THEMES_PATH.'/'.$this->themeName;
			else
				$basePath = $CONF_THEMES_PATH.'/'.$this->themeName;

			$filename = $basePath.$THEMES_TEMPLATE_DIR.'/'.$file;

			if ( $this->debug )
				echo "Opening file '$filename' ... <br>\n";


			// Fail if the template file is not found
			if ( !@is_file( $filename ) )
			{
				if ( $this->debug )
					// Debug mode is ON
					// Display can't open the template file message
					echo "Error!! Can not open the file '".$filename." on line 178'. <br>\n";
			}
			else
			{
				// ------------------------------------------------------------------- //
				// Read the content of template file
				// ------------------------------------------------------------------- //
				$fp = @fopen( $filename, 'r' );
				$templateContent = fread ( $fp, filesize ($filename) );
				fclose ($fp);
				// ------------------------------------------------------------------- //


				// ------------------------------------------------------------------- //
				// Replace the file link in the template file.
				// Use the php program 'file.php' to get the content of the file file
				// (e.g. <IMG SRC="xx">, <SCRIPT... SRC=""></SCRIPT>, <..BACKGROUND=""> )
				// *** Increase the security ***
				// ------------------------------------------------------------------- //
				$templateArray = explode( "\n", $templateContent );
				$rownum = 0;
				$rowName = array();
				for ( $i=0; $i<count($templateArray); $i++ )
				{
					if ( eregi( 'src="(.*)"', $templateArray[$i], $filePath) ||
						 eregi( 'background="(.*)"', $templateArray[$i], $filePath) )
					{
						// Split the string by '"'
						$filePath = explode( '"', $filePath[1] );

						// Stop if the path is NULL
						if ( !$filePath[0] )
							break;

						// Check the file type
						$strLength = strlen( $filePath[0] );
						if ( 0==strcmp( strtoupper('.js'), strtoupper(substr($filePath[0], $strLength - 3, $strLength))) )
							// The file link is javascript, so the path will different
							$fileType = 'javascript';
						else
							// no default file type
							$fileType = '';

						if ( $fileType )
							// If the file link is set
							$fileLink = $filePath[0];
						else
							// No file link is set, link to default template directory
							$fileLink = 'themes/'.$this->themeName.$THEMES_TEMPLATE_DIR.'/'.$filePath[0];


						// Replace each line one by one if the line content the file link ('src=', or 'background=')
						if ( $this->speedLimit )
							// Have set the download speed limit
							@ $templateArray[$i] = ereg_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&file='.$fileLink, $templateArray[$i] );
						else
							// No download speed limit
							@ $templateArray[$i] = ereg_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&nolimit=&file='.$fileLink, $templateArray[$i] );
					}

					// Get the name of each 'ROW' block, if existed.
					if ( eregi( '<!-- \[ROW NAME="(.*)"\] -->', $templateArray[$i], $rowname) )
					{
						$rowname            = explode( '"', $rowname[1] );
						$rowName[$rownum++] = $rowname[0];
					}
				}
				// Join the result together
				$templateContent = implode ( "\n", $templateArray );

				// Remove the template array from memory
				unset ($templateArray);
				// -------------------- End of replace image link --------------------- //


				// ------------------------------------------------------------------- //
				// Include the themes language file
				// ------------------------------------------------------------------- //
				GLOBAL $THEMES_DEFAULT_LANG;

				// Stop when the user selected language file readed
				if ( !@include( $this->_langPath = $basePath.$THEMES_LANG_DIR.'/'.$this->themeLang.'.lang.php' ) ) 
				{
					// Trying load the themes default language file
					if ( !@include( $this->_langPath = $basePath.$THEMES_LANG_DIR.'/'.$THEMES_DEFAULT_LANG.'.lang.php' ) )
					{
						// Trying load default 'english' language file
						if ( !@include( $this->_langPath = $basePath.$THEMES_LANG_DIR.'/en.lang.php' ) )
						{
							if ( $this->debug )
								// Display the can't open language file error message when debug mode is ON
								echo "Can not open the language file '".$basePath.$THEMES_LANG_DIR.'/'.$this->themeLang.'.lang.php'."'.<br>\n";

							exit;
						}
					}
				}
				// ------------------------------------------------------------------- //


				// ------------------------------------------------------------------- //
				// Find the key of string from the language file
				// ------------------------------------------------------------------- //
				$langString = array_keys($LANG);
				for ( $i=0; $i<count($langString); $i++ )
				{
					@ $templateContent = ereg_replace( '\['.$langString[$i].'\]', $LANG[$langString[$i]], $templateContent );
				}
				// ------------------------------------------------------------------- //


				// Store all block of 'ROW' into class variable
				foreach ($rowName as $rowname)
				{
					eregi( '<!-- \[ROW NAME="'.$rowname.'"\] -->(.*)<!-- \[ENDROW NAME="'.$rowname.'"\] -->', $templateContent, $block );
					$block = eregi_replace( '<!-- \[ROW NAME="'.$rowname.'"\] -->', '', $block[0]);
					$block = eregi_replace( '<!-- \[ENDROW NAME="'.$rowname.'"\] -->', '', $block);
					$this->_tempResult[$rowname] = $block;
				}


				// Return the temporary result
				$this->_result = $templateContent;


				// Show the HTML code when in debug mode
				if ( $this->debug )
				{
					echo "<br>\n";
					echo "DEBUG : Showing the source code of temnplate file. (finish the replacement of language)<br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
					echo " Template file : '$filename' <br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
					echo " Dumping the source code....... <br>\n";
					echo highlight_string($this->_result, TRUE);
					echo "<br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
					echo " End of dumping the template source code! <br>\n";
					echo "---------------------------------------------------------------------------------------------<br>\n";
				}
			}
		}
		// -- End of class function :: getTemplate(..) --

	

		// ===================================================================
		// == Class function    myTheme::assign( $pattern='', $replace='' );==
		// ==                                                               ==
		// == Replace the patten string with the dynamic conten. This is    ==
		// == very important to replace the dynamic content into the        ==
		// == template layout.                                              ==
		// ==                                                               ==
		// == Usage : $myfile->assign( 'DYNAMIC_VAR', 'some data' );        ==
		// ===================================================================
		function assign( $pattern='', $replace='' )
		{
			// -------------------------------------------------------------------- //
			// Do the replacement if the input '$replace' is a refer as a file link.
			// (e.g. <IMG SRC="image.jpg" .....> )
			// -------------------------------------------------------------------- //
			if ( eregi( 'src="(.*)"', $replace, $filePath) ||
				 eregi( 'background="(.*)"', $replace, $filePath) )
			{
				// Split the string by '"'
				$filePath = explode( '"', $filePath[1] );

				// Stop if the path is NULL
				if ( !$filePath[0] )
					break;

				// Check the file type
				$strLength = strlen( $filePath[0] );
				if ( 0==strcmp( strtoupper('.js'), strtoupper(substr($filePath[0], $strLength - 3, $strLength))) )
					// The file link is javascript, so the path will different
					$fileType = 'javascript';
				else
					// no default file type
					$fileType = '';

				if ( $fileType )
					// If the file link is set
					$fileLink = $filePath[0];
				else
					// No file link is set, link to default template directory
					$fileLink = 'themes/'.$this->themeName.$THEMES_TEMPLATE_DIR.'/'.$filePath[0];


				// Replace each line one by one if the line content the file link ('src=', or 'background=')
				if ( $this->speedLimit )
					// Have set the download speed limit
					@ $replace = ereg_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&file='.$fileLink, $replace );
				else
					// No download speed limit
					@ $replace = ereg_replace( $filePath[0], DEFAULT_file_php.'?type='.$fileType.'&mode=load&nolimit=&file='.$fileLink, $replace );
			}
			// -------------------- End of replace image link --------------------- //


			// Check the dynamic assign is it a special data
			// If the replace string is the form of 'LANG=XXXX', this template
			// system will try to retrival the string from the language file
			// which the text is named by 'XXXX'.
			$langText = explode( 'LANG=', $replace);
			if (( $langText[1] ) && ( @include( $this->_langPath ) ) )
				// Special replacement from Language file
				$this->_result = ereg_replace( '\{'.$pattern.'\}', $LANG[$langText[1]], $this->_result );
			else
				// Replace the pattern with the dynamic data.
				$this->_result = ereg_replace( '\{'.$pattern.'\}', $replace, $this->_result );


			// Display the debug content
			if ( $this->debug )
			{
				echo "<br>\n";
				echo "DEBUG : assign ( '$pattern', '$replace' );<br>\n";
				echo "DEBUG : '{$pattern}' => '$replace'<br>\n";
			}
		}
		// -- End of class function :: assign(..) --


	
		// ===================================================================
		// == Class function    myTheme::repeatRow( $rowname );             ==
		// ==                                                               ==
		// == Place an un-assigned blcok of row into apporate place of      ==
		// == template. (i.e. empty row )                                   ==
		// ==                                                               ==
		// == Usage : $myfile->repeatRow( 'ROW1' );                         ==
		// ===================================================================
		function repeatRow( $rowname )
		{
			// Add an empty block at the end of the current block of element
			$this->_result = eregi_replace( '<!-- \[ENDROW NAME="'.$rowname.'"\] -->', 
 				                            $this->_tempResult[$rowname].'<!-- [ENDROW NAME="'.$rowname.'"] -->',
				                            $this->_result );
		}
		// -- End of class function :: repeatRow(..) --


		// ===================================================================
		// == Class function    myTheme::getHeader();                       ==
		// ==                                                               ==
		// == Display the HTML header. This will set the TITLE, css style   ==
		// == sheel, language and also the javascript.                      ==
		// ==                                                               ==
		// == Usage : $myfile->getHeader();                                 ==
		// ===================================================================
		function getHeader()
		{
			GLOBAL 
				$THEMES_CSS_FILE,
				$CONF_SERVER_NAME;

			// Include the language file again
			@ include( $this->_langPath );

			// send the current charset to the client browser
			header ('Content-Type: text/html; charset='.$LANG_CHARSET);

			echo "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n";
			echo "<html>\n";
			echo "<head>\n";
			echo "<title>".$CONF_SERVER_NAME."</title>\n";
			echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=".$LANG_CHARSET."\">\n";
			echo "<meta http-equiv=\"Content-Language\" content=\"".$CONTENT_LANG."\">\n";

			// Show the CSS style sheet if the browser is support
			if ( cssSupport() && $THEMES_CSS_FILE )
			{
				$cssFile = 'themes/'.$this->themeName.$THEMES_CSS_FILE;
				echo "<link rel=\"stylesheet\" href=\"".DEFAULT_file_php."?&mode=load&nolimit=&file=".$cssFile."\">\n";
			}

			echo "<SCRIPT LANGUAGE=\"JavaScript\" src=\"".DEFAULT_file_php."?type=javascript&mode=load&nolimit=&file=script.js\"></script>\n";
			echo "</head>\n";
		}
		// -- End of class function :: getHeader(..) --


		// ===================================================================
		// == Class private function	myTheme::_checkTheme();				==
		// ==																==
		// == Return TRUE if the themes can be use. Otherwise the themes is ==
		// == disabled and can not be use.                                  ==
		// == Usage : $result = _checkTheme();								==
		// ===================================================================
		function _checkTheme()
		{
			GLOBAL
				$CONF_ADMIN_THEMES_PATH,
				$THEMES_ENABLE,
				$THEMES_NAME,
				$THEMES_IMAGE_DIR,
				$THEMES_LANG_DIR,
				$THEMES_TEMPLATE_DIR,
				$THEMES_DEFAULT_LANG,
				$THEMES_CSS_FILE;

			if ( !@include ( $CONF_ADMIN_THEMES_PATH.'/'.$this->themeName.'/themes.php' ) )
				return false;
			
			if  ( !$THEMES_ENABLE )
				return false;
			else
				return true;

		}
		// -- End of class private function :: _checkTheme(..) --
	}
	// -- End of class :: myTheme --



	function chkTheme( $userDefine='', $systemDefault='' )
	{
		GLOBAL
			$CONF_ADMIN_THEMES_PATH,
			$THEMES_ENABLE,
			$THEMES_NAME,
			$THEMES_IMAGE_DIR,
			$THEMES_LANG_DIR,
			$THEMES_TEMPLATE_DIR,
			$THEMES_DEFAULT_LANG,
			$THEMES_CSS_FILE;


		if ( $userDefine )
			if ( @include ($CONF_ADMIN_THEMES_PATH.'/'.$userDefine.'/themes.php') )
				if ( $THEMES_ENABLE ) return $userDefine;

		if ( $systemDefault )
			if ( @include ($CONF_ADMIN_THEMES_PATH.'/'.$systemDefault.'/themes.php' ) )
				if ( $THEMES_ENABLE ) return $systemDefault;

		if ( @include ($CONF_ADMIN_THEMES_PATH.'/OES.default/themes.php' ) )
			if ( $THEMES_ENABLE ) return 'OES.default';

		return false;
	}
?>